@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: dlopen.c,v 1.13 2003/01/21 17:12:22 grubba Exp $
+|| $Id: dlopen.c,v 1.73 2003/08/22 16:02:34 tomas Exp $
 */
 
 #include <global.h>
@@ -16,7 +16,10 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <errno.h>
+#if !defined(__FreeBSD__) && !defined(__OpenBSD__)
+/* FreeBSD and OpenBSD has <malloc.h>, but it just contains a warning... */
 #include <malloc.h>
+#endif /* !__FreeBSD__ && !__OpenBSD__ */
 #include <windows.h>
 #include <memory.h>
 #include <sys/stat.h>
@@ -199,7 +202,7 @@ size_t STRNLEN(char *s, size_t maxlen)
 
 #else /* PIKE_CONCAT */
 
-RCSID("$Id: dlopen.c,v 1.13 2003/01/21 17:12:22 grubba Exp $");
+RCSID("$Id: dlopen.c,v 1.73 2003/08/22 16:02:34 tomas Exp $");
 
 #endif
 
@@ -2523,6 +2526,10 @@ static void init_dlopen(void)
     EXPORT(fseek);
     EXPORT(fread);
     EXPORT(strtol);
+    EXPORT(rewind);
+    EXPORT(fputs);
+    EXPORT(freopen);
+    EXPORT(memcmp);
   }
 #if 0
 #ifdef _M_IA64
@@ -2574,15 +2581,24 @@ static void init_dlopen(void)
 #endif
 }
 
-#ifdef HAVE__ALLDIV
 /* Strap to load symbols that might be needed from modules. */
+#ifdef HAVE__ALLDIV
 void _alldiv(void);
+#endif
+#ifdef HAVE__AULLSHR
+void _aullshr(void);
+#endif
 void dl_dummy(void)
 {
+#ifdef HAVE__ALLDIV
   _alldiv();
-}
 #endif
 
+#ifdef HAVE__AULLSHR
+  _aullshr();
+#endif
+}
+
 /****************************************************************/
 
 #ifdef TEST
