@@ -1,18 +1,17 @@
 /*
- * $Id: fdlib.h,v 1.38 2000/12/23 07:33:49 hubbe Exp $
- */
+|| This file is part of Pike. For copyright information see COPYRIGHT.
+|| Pike is distributed under GPL, LGPL and MPL. See the file COPYING
+|| for more information.
+|| $Id: fdlib.h,v 1.50 2003/12/17 21:27:17 marcus Exp $
+*/
+
 #ifndef FDLIB_H
 #define FDLIB_H
 
 #include "global.h"
 #include "pike_macros.h"
 
-
-/* The sp macro conflicts with Solaris 2.5.1's <sys/file.h>. */
-#ifdef sp
-#undef sp
-#define STACKPOINTER_WAS_DEFINED
-#endif /* sp */
+#include "pike_netlib.h"
 
 #ifdef HAVE_ERRNO_H
 #include <errno.h>
@@ -38,17 +37,14 @@
 #include <socket.h>
 #endif /* HAVE_SOCKET_H */
 
-/* Restore the sp macro */
-#ifdef STACKPOINTER_WAS_DEFINED
-#define sp Pike_sp
-#undef STACK_POINTER_WAS_DEFINED
-#endif /* STACKPOINTER_WAS_DEFINED */
+#include "pike_netlib.h"
 
 #define fd_INTERPROCESSABLE   1
 #define fd_CAN_NONBLOCK       2
 #define fd_CAN_SHUTDOWN       4
 #define fd_BUFFERED           8
 #define fd_BIDIRECTIONAL     16
+#define fd_REVERSE	     32
 
 
 #ifdef HAVE_WINSOCK_H
@@ -64,11 +60,18 @@
 #define FD_SETSIZE MAX_OPEN_FILEDESCRIPTORS
 #endif
 
-#include <winsock.h>
 #include <winbase.h>
 
 typedef int FD;
 
+#if _INTEGRAL_MAX_BITS >= 64
+typedef struct _stati64 PIKE_STAT_T;
+typedef __int64 PIKE_OFF_T;
+#else
+typedef struct stat PIKE_STAT_T;
+typedef off_t PIKE_OFF_T;
+#endif
+
 #define SOCKFUN1(NAME,T1) PMOD_EXPORT int PIKE_CONCAT(debug_fd_,NAME) (FD,T1);
 #define SOCKFUN2(NAME,T1,T2) PMOD_EXPORT int PIKE_CONCAT(debug_fd_,NAME) (FD,T1,T2);
 #define SOCKFUN3(NAME,T1,T2,T3) PMOD_EXPORT int PIKE_CONCAT(debug_fd_,NAME) (FD,T1,T2,T3);
@@ -116,8 +119,8 @@ PMOD_EXPORT char *debug_fd_info(int fd);
 PMOD_EXPORT int debug_fd_query_properties(int fd, int guess);
 void fd_init();
 void fd_exit();
-PMOD_EXPORT int debug_fd_stat(char *file, struct stat *buf);
-PMOD_EXPORT FD debug_fd_open(char *file, int open_mode, int create_mode);
+PMOD_EXPORT int debug_fd_stat(const char *file, PIKE_STAT_T *buf);
+PMOD_EXPORT FD debug_fd_open(const char *file, int open_mode, int create_mode);
 PMOD_EXPORT FD debug_fd_socket(int domain, int type, int proto);
 PMOD_EXPORT int debug_fd_pipe(int fds[2] DMALLOC_LINE_ARGS);
 PMOD_EXPORT FD debug_fd_accept(FD fd, struct sockaddr *addr, ACCEPT_SIZE_T *addrlen);
@@ -136,10 +139,10 @@ SOCKFUN1(listen, int)
 PMOD_EXPORT int debug_fd_close(FD fd);
 PMOD_EXPORT ptrdiff_t debug_fd_write(FD fd, void *buf, ptrdiff_t len);
 PMOD_EXPORT ptrdiff_t debug_fd_read(FD fd, void *to, ptrdiff_t len);
-PMOD_EXPORT ptrdiff_t debug_fd_lseek(FD fd, ptrdiff_t pos, int where);
-PMOD_EXPORT int debug_fd_ftruncate(FD fd, ptrdiff_t len);
+PMOD_EXPORT PIKE_OFF_T debug_fd_lseek(FD fd, PIKE_OFF_T pos, int where);
+PMOD_EXPORT int debug_fd_ftruncate(FD fd, PIKE_OFF_T len);
 PMOD_EXPORT int debug_fd_flock(FD fd, int oper);
-PMOD_EXPORT int debug_fd_fstat(FD fd, struct stat *s);
+PMOD_EXPORT int debug_fd_fstat(FD fd, PIKE_STAT_T *s);
 PMOD_EXPORT int debug_fd_select(int fds, FD_SET *a, FD_SET *b, FD_SET *c, struct timeval *t);
 PMOD_EXPORT int debug_fd_ioctl(FD fd, int cmd, void *data);
 PMOD_EXPORT FD debug_fd_dup(FD from);
@@ -176,6 +179,7 @@ PMOD_EXPORT FD debug_fd_dup2(FD from, FD to);
 #define fd_TRUNC 16
 #define fd_EXCL 32
 #define fd_BINARY 0
+#define fd_LARGEFILE 0
 
 #define fd_shutdown_read SD_RECEIVE
 #define fd_shutdown_write SD_SEND
@@ -201,7 +205,7 @@ struct my_fd_set_s
 typedef struct my_fd_set_s my_fd_set;
 
 #ifdef PIKE_DEBUG
-#define fd_check_fd(X) do { if(fd_type[X]>=0) fatal("FD_SET on closed fd %d (%d) %s:%d.\n",X,da_handle[X],__FILE__,__LINE__); }while(0)
+#define fd_check_fd(X) do { if(fd_type[X]>=0) Pike_fatal("FD_SET on closed fd %d (%d) %s:%d.\n",X,da_handle[X],__FILE__,__LINE__); }while(0)
 #else
 #define fd_check_fd(X)
 #endif
@@ -289,6 +293,8 @@ PMOD_EXPORT void closedir(DIR *dir);
 
 
 typedef int FD;
+typedef struct stat PIKE_STAT_T;
+typedef off_t PIKE_OFF_T;
 
 #define fd_info(X) ""
 #define fd_init()
@@ -302,6 +308,11 @@ typedef int FD;
 #define fd_TRUNC O_TRUNC
 #define fd_EXCL O_EXCL
 #define fd_BINARY 0
+#ifdef O_LARGEFILE
+#define fd_LARGEFILE O_LARGEFILE
+#else /* !O_LARGEFILE */
+#define fd_LARGEFILE 0
+#endif /* O_LARGEFILE */
 
 #define fd_query_properties(X,Y) ( fd_INTERPROCESSABLE | (Y))
 
@@ -393,7 +404,9 @@ typedef struct my_fd_set_s my_fd_set;
    MEMCPY((TO),&(FROM)->tmp,sizeof(*(TO)))
 
 #define FILE_CAPABILITIES (fd_INTERPROCESSABLE | fd_CAN_NONBLOCK)
+#ifndef __amigaos__
 #define PIPE_CAPABILITIES (fd_INTERPROCESSABLE | fd_BUFFERED | fd_CAN_NONBLOCK)
+#endif
 #define UNIX_SOCKET_CAPABILITIES (fd_INTERPROCESSABLE | fd_BIDIRECTIONAL | fd_CAN_NONBLOCK)
 #define SOCKET_CAPABILITIES (fd_INTERPROCESSABLE | fd_BIDIRECTIONAL | fd_CAN_NONBLOCK | fd_CAN_SHUTDOWN)
 
