@@ -2,11 +2,11 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: interpret.c,v 1.350 2004/05/21 19:02:49 grubba Exp $
+|| $Id: interpret.c,v 1.353 2004/09/30 15:28:08 mast Exp $
 */
 
 #include "global.h"
-RCSID("$Id: interpret.c,v 1.350 2004/05/21 19:02:49 grubba Exp $");
+RCSID("$Id: interpret.c,v 1.353 2004/09/30 15:28:08 mast Exp $");
 #include "interpret.h"
 #include "object.h"
 #include "program.h"
@@ -63,8 +63,16 @@ RCSID("$Id: interpret.c,v 1.350 2004/05/21 19:02:49 grubba Exp $");
 
 /* Keep some margin on the stack space checks. They're lifted when
  * handle_error runs to give it some room. */
-#define SVALUE_STACK_MARGIN 100	/* Tested in 7.1: 40 was enough, 30 wasn't. */
-#define C_STACK_MARGIN 8000	/* Tested in 7.1: 3000 was enough, 2600 wasn't. */
+/* Observed in 7.1: 40 was enough, 30 wasn't. */
+#define SVALUE_STACK_MARGIN (100 + LOW_SVALUE_STACK_MARGIN)
+/* Observed in 7.4: 11000 was enough, 10000 wasn't. */
+#define C_STACK_MARGIN (20000 + LOW_C_STACK_MARGIN)
+
+/* Another extra margin to use while dumping the raw error in
+ * exit_on_error, so that backtrace_frame._sprintf can be called
+ * then. */
+#define LOW_SVALUE_STACK_MARGIN 20
+#define LOW_C_STACK_MARGIN 500
 
 #ifdef HAVE_COMPUTED_GOTO
 PIKE_OPCODE_T *fcode_to_opcode = NULL;
@@ -159,7 +167,8 @@ void gc_mark_stack_external (struct pike_frame *f,
 
 static void gc_check_stack_callback(struct callback *foo, void *bar, void *gazonk)
 {
-  gc_mark_stack_external (Pike_fp, Pike_sp, Pike_interpreter.evaluator_stack);
+  if (Pike_interpreter.evaluator_stack)
+    gc_mark_stack_external (Pike_fp, Pike_sp, Pike_interpreter.evaluator_stack);
 }
 #endif
 
@@ -555,7 +564,14 @@ PMOD_EXPORT void find_external_context(struct external_variable_context *loc,
     Pike_fatal("No object\n");
 #endif
 
-  p = loc->o->prog;
+  if (!(p = loc->o->prog)) {
+    /* magic fallback */
+    p = get_program_for_object_being_destructed(loc->o);
+    if(!p)
+    {
+      Pike_error("Cannot access parent of destructed object.\n");
+    }
+  }
 
 #ifdef DEBUG_MALLOC
   if (loc->o->refs == 0x55555555) {
@@ -590,7 +606,7 @@ PMOD_EXPORT void find_external_context(struct external_variable_context *loc,
 
     TRACE((4,"-   o->parent_identifier=%d inherit->identifier_level=%d\n",
 	   (p->flags & PROGRAM_USES_PARENT) ?
-	   PARENT_INFO(loc->o)->parent_identifier : -1,
+	   LOW_PARENT_INFO(loc->o, p)->parent_identifier : -1,
 	   inh->identifier_level));
 
     switch(inh->parent_offset)
@@ -1979,7 +1995,12 @@ void unlink_previous_frame(void)
 
 void mega_apply(enum apply_type type, INT32 args, void *arg1, void *arg2)
 {
-  check_c_stack(8192);
+  /* The C stack margin is normally 8 kb, but if we get here during a
+   * lowered margin then don't fail just because of that, unless it's
+   * practically zero. */
+  check_c_stack(Pike_interpreter.c_stack_margin ?
+		Pike_interpreter.c_stack_margin : 100);
+
   if(low_mega_apply(type, args, arg1, arg2))
   {
     eval_instruction(Pike_fp->pc
@@ -2077,11 +2098,11 @@ PMOD_EXPORT void call_handle_error(void)
 {
   dmalloc_touch_svalue(&throw_value);
 
-  if (Pike_interpreter.svalue_stack_margin) {
+  if (Pike_interpreter.svalue_stack_margin > LOW_SVALUE_STACK_MARGIN) {
     int old_t_flag = Pike_interpreter.trace_level;
     Pike_interpreter.trace_level = 0;
-    Pike_interpreter.svalue_stack_margin = 0;
-    Pike_interpreter.c_stack_margin = 0;
+    Pike_interpreter.svalue_stack_margin = LOW_SVALUE_STACK_MARGIN;
+    Pike_interpreter.c_stack_margin = LOW_C_STACK_MARGIN;
     *(Pike_sp++) = throw_value;
     dmalloc_touch_svalue(Pike_sp-1);
     throw_value.type=T_INT;
