@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: module.c,v 1.20 2002/10/11 01:39:33 nilsson Exp $
+|| $Id: module.c,v 1.27 2004/04/15 13:20:19 grubba Exp $
 */
 
 #include "global.h"
@@ -17,15 +17,21 @@
 #include "object.h"
 #include "mapping.h"
 #include "program_id.h"
-#include "language.h"
 #include "lex.h"
 
 #include "modules/modlist_headers.h"
-#ifndef IN_TPIKE
 #include "post_modules/modlist_headers.h"
-#endif
 
-RCSID("$Id: module.c,v 1.20 2002/10/11 01:39:33 nilsson Exp $");
+RCSID("$Id: module.c,v 1.27 2004/04/15 13:20:19 grubba Exp $");
+
+/* Define this to trace the initialization and cleanup of static modules. */
+/* #define TRACE_MODULE */
+
+#if defined(TRACE_MAIN) || defined(TRACE_MODULE)
+#define TRACE(X)	fprintf X
+#else /* !TRACE_MAIN */
+#define TRACE(X)
+#endif /* TRACE_MAIN */
 
 typedef void (*modfun)(void);
 
@@ -39,15 +45,12 @@ struct static_module
 static struct static_module module_list[] = {
   { "Builtin", low_init_main, low_exit_main }
 #include "modules/modlist.h"
-#ifndef IN_TPIKE
 #include "post_modules/modlist.h"
-#endif
-  ,{ "Builtin2", init_main, exit_main }
 };
 
 void init_modules(void)
 {
-  struct program *p;
+  struct program *p = NULL;
   unsigned int e;
   struct lex save_lex;
 
@@ -61,34 +64,61 @@ void init_modules(void)
   for(e=0;e<NELEM(module_list);e++)
   {
     JMP_BUF recovery;
-    start_new_program();
+    if (!p) {
+      start_new_program();
+      p = Pike_compiler->new_program;
+    }
     if(SETJMP(recovery)) {
+      /* FIXME: We could loop here until we find p. */
       free_program(end_program());
+      p = NULL;
       call_handle_error();
     } else {
+      TRACE((stderr, "Initializing static module #%d: \"%s\"...\n",
+	     e, module_list[e].name));
       module_list[e].init();
-      debug_end_class(module_list[e].name,strlen(module_list[e].name),0);
+      if (
+#if 0
+	  Pike_compiler->new_program->num_identifier_references
+#else /* !0 */
+	  1
+#endif /* 0 */
+	  ) {
+	debug_end_class(module_list[e].name,strlen(module_list[e].name),0);
+	p = NULL;
+      } else {
+	/* No identifier references -- Disabled module. */
+      }
     }
     UNSETJMP(recovery);
   }
+  if (p) free_program(end_program());
   push_text("_static_modules");
   push_object(low_clone(p=end_program()));
   f_add_constant(2);
   free_program(p);
   free_string(lex.current_file);
   lex = save_lex;
+
+  init_main();
 }
 
 void exit_modules(void)
 {
   JMP_BUF recovery;
   int e;
+
+  exit_main();
+
   for(e=NELEM(module_list)-1;e>=0;e--)
   {
     if(SETJMP(recovery))
       call_handle_error();
-    else
+    else {
+      TRACE((stderr, "Exiting static module #%d: \"%s\"...\n",
+	     e, module_list[e].name));
       module_list[e].exit();
+    }
     UNSETJMP(recovery);
   }
 }
