@@ -1,4 +1,11 @@
-   COLOR("red"                 ,255,0,0)
+/*
+|| This file is part of Pike. For copyright information see COPYRIGHT.
+|| Pike is distributed under GPL, LGPL and MPL. See the file COPYING
+|| for more information.
+|| $Id: colors.h,v 1.8 2002/10/11 01:39:41 nilsson Exp $
+*/
+
+COLOR("red"                 ,255,0,0)
    COLOR("black"               ,0,0,0)
    COLOR("blue"                ,0,0,255)
    COLOR("cyan"                ,0,255,255)
