@@ -1,229 +1,27 @@
+/*
+|| This file is part of Pike. For copyright information see COPYRIGHT.
+|| Pike is distributed under GPL, LGPL and MPL. See the file COPYING
+|| for more information.
+|| $Id: layers.c,v 1.98 2006/03/08 01:21:35 peter Exp $
+*/
+
 /*
 **! module Image
-**! note
-**!	$Id: layers.c,v 1.64 2004/02/03 20:45:25 grubba Exp $
 **! class Layer
 **! see also: layers
-**!
-
-**!
-**!
-**! <add_appendix name="layers" title="Image.Layer modes">
-**! <dl compact>
-**! <dt><i>The variables in the expression:</i>
-**! <dt>L<dd><i>The active layer</i>
-**! <dt>S<dd><i>The source layer (the sum of the layers below)</i>
-**! <dt>D<dd><i>The destintion layer (the result)</i>
-**! <dt>Xrgb<dd><i>Layer red (<b>Xr</b>), green (<b>Xg</b>) or blue channel (<b>Xb</b>) </i>
-**! <dt>Xhsv<dd><i>Layer hue (<b>Xh</b>), saturation (<b>Xs</b>) or value channel (<b>Xv</b>) (virtual channels)</i>
-**! <dt>Xhls<dd><i>Layer hue (<b>Xh</b>), lightness channel (<b>Xl</b>) or saturation (<b>Xs</b>) (virtual channels)</i>
-**! <dt>aX<dd><i>Layer alpha, channel in layer alpha</i>
-**! </dl>
-**! <i>All channels are calculated separately, if nothing else is specified.</i>
-**! <execute>
-**! import Image;
-**!
-**! void write_image(string desc,
-**! 		     string filename,
-**! 		     Image img,
-**!                  string longdesc)
-**! {
-**!    begin_tag("tr");
-**!    write(mktag("td",(["align":"right","valign":"center"]),
-**!          mktag("b",0,desc)));
-**!    write(mktag("td",(["align":"right"]),illustration_jpeg(img,(["dpi":150.0]))));
-**!    write(mktag("td",(["align":"left","valign":"center"]),longdesc));
-**           (replace(longdesc,({",",";",")"}),
-**                    ({",<wbr>",";<wbr>",")<wbr>"}))/
-**            "<wbr>")/1*({mktag("wbr")}) ) );
-**!    write(end_tag());
-**! }
-**!
-**! int main()
-**! {
-**!    object ltrans=Layer((["image":
-**! 			     Image(32,32,160,160,160)->
-**! 			     box(0,0,15,15,96,96,96)->
-**! 			     box(16,16,31,31,96,96,96)->scale(0.5),
-**! 			     "tiled":1,
-**! 			     "mode":"normal"]));
-**!
-**!    object circle=load("circle50.pnm");
-**!    object image_test=load("image_ill.pnm");
-**!    object ryoki=load_layer("ryoki_carrot.png");
-**!
-**!    object lc1=
-**! 	     Layer((["image":circle->clear(255,0,0),
-**! 		     "alpha":circle,
-**! 		     "xoffset":5,
-**! 		     "yoffset":5]));
-**!
-**!    object lc2=
-**! 	  Layer((["image":circle->clear(0,0,255),
-**! 		  "alpha":circle,
-**! 		  "xoffset":25,
-**! 		  "yoffset":25]));
-**!    object lc2b=
-**! 	  Layer((["image":circle,
-**! 		  "alpha":circle*({0,0,255}),
-**! 		  "xoffset":25,
-**! 		  "yoffset":25]));
-**!
-**!    object lr1=
-**! 	  Layer((["image":circle->test(63)->scale(1.25),
-**! 		  "alpha":circle->scale(1.25),
-**! 		  "xoffset":10,
-**! 		  "yoffset":5]));
-**!    object lr2=
-**! 	  Layer((["image":ryoki->image(),
-**! 		  "alpha":ryoki->alpha(),
-**! 		  "xoffset":5,
-**! 		  "yoffset":20]));
-**!
-**!    object li1=
-**! 	  Layer((["image":image_test->scale(80,0),
-**! 		  "mode":"normal",
-**! 		  "xoffset":0,
-**! 		  "yoffset":0]));
-**!
-**!    object li2=
-**! 	  lay(
-**! 	     ({
-**! 	     (["image":circle->clear(255,0,0),"alpha":circle,
-**! 	       "xoffset":5,"yoffset":0,"mode":"add"]),
-**! 	     (["image":circle->clear(0,255,0),"alpha":circle,
-**! 	       "xoffset":25,"yoffset":0,"mode":"add"]),
-**! 	     (["image":circle->clear(0,0,255),"alpha":circle,
-**! 	       "xoffset":15,"yoffset":20,"mode":"add"]),
-**! 	     (["image":circle->clear(0,0,0)->scale(0.5),
-**! 	       "alpha":circle->scale(0.5),
-**! 	       "xoffset":0,"yoffset":55]),
-**! 	     (["image":circle->clear(255,255,255)->scale(0.5),
-**! 	       "alpha":circle->scale(0.5),
-**! 	       "xoffset":55,"yoffset":55])
-**! 	     }));
-**!
-**!    Layer li2b=Layer(li2->image()->clear(255,255,255),li2->image());
-**!
-**!    object lzo0=
-**! 	  lay( ({
-**! 	     Layer(Image(4,10)
-**! 		   ->tuned_box(0,0,3,8,({({255,255,255}),({255,255,255}),
-**! 					 ({0,0,0}),({0,0,0})}))
-**! 		   ->scale(40,80))
-**! 	     ->set_offset(0,0),
-**! 	     Layer(Image(40,80)
-**! 		   ->tuned_box(0,0,40,78,({({255,255,255}),({255,255,255}),
-**! 					  ({0,0,0}),({0,0,0})})))
-**! 	     ->set_offset(40,0),
-**! 	     Layer(Image(80,80)
-**! 		   ->tuned_box(0,0,80,80,({({255,0,0}),({255,255,0}),
-**! 					  ({0,0,255}),({0,255,0})})))
-**! 	     ->set_offset(80,0),
-**! 	  }) );
-**!
-**!    object scale=
-**! 	  Image(4,80)
-**! 	  ->tuned_box(0,0,40,78,({({255,255,255}),({255,255,255}),
-**! 				  ({0,0,0}),({0,0,0})}));
-**!    object lzo1=
-**! 	  lay( ({
-**! 	     Layer(scale)->set_offset(2,0),
-**! 	     Layer(scale->invert())->set_offset(6,0),
-**! 	     Layer(Image(26,80)->test())->set_offset(12,0),
-**! 	     Layer(scale)->set_offset(42,0),
-**! 	     Layer(scale->invert())->set_offset(46,0),
-**! 	     Layer(Image(26,80)->test())->set_offset(52,0),
-**! 	     Layer(scale)->set_offset(82,0),
-**! 	     Layer(scale->invert())->set_offset(86,0),
-**! 	     Layer(Image(26,80)->test())->set_offset(92,0),
-**! 	     Layer(scale)->set_offset(122,0),
-**! 	     Layer(scale->invert())->set_offset(126,0),
-**! 	     Layer(Image(26,80,"white"),
-**! 		   Image(26,80)->test())->set_offset(132,0),
-**! 	  }));
-**!
-**!    object lca1;
-**!
-**!    object a=
-**! 	  lay( ({ lca1=lay(({Layer((["fill":"white"])),
-**!                          lc1}),0,0,80,80),
-**! 		  lay(({lc1}),0,0,80,80)->set_offset(80,0),
-**! 		  lay(({li1}),0,0,80,80)->set_offset(160,0),
-**! 		  lay(({li1}),0,0,80,80)->set_offset(240,0),
-**! 		  lay(({lr1}),0,0,80,80)->set_offset(320,0),
-**! 		  lzo0->set_offset(400,0)}),
-**! 	       0,0,560,80);
-**!
-**!    object b=
-**! 	  lay( ({ lay(({lc2}),0,0,80,80),
-**! 		  lay(({lc2b}),0,0,80,80)->set_offset(80,0),
-**! 		  lay(({li2}),0,0,80,80)->set_offset(160,0),
-**! 		  lay(({li2b}),0,0,80,80)->set_offset(240,0),
-**! 		  lay(({lr2}),0,0,80,80)->set_offset(320,0),
-**! 		  lzo1->set_offset(400,0)}),
-**! 	       0,0,560,80);
-**!
-**    xv(a); xv(b);
-**!
-**!    begin_tag("table",(["cellspacing":"0","cellpadding":"1"]));
-**!
-**! //    write_image("top layer image","bi",b->image());
-**! //    write_image("top layer alpha","ba",b->alpha());
-**!
-**! //    write_image("bottom layer image","ai",a->image());
-**! //    write_image("bottom layer alpha","aa",b->alpha());
-**!
-**!    write_image("top layer","b",lay(({ltrans,b}))->image(),
-**!		   "");
-**!    write_image("bottom layer","a",lay(({ltrans,a}))->image(),
-**!                "");
-**!
-**!    write(mktag("tr",0,mktag("td",0,"\240")));
-**!
-**!    foreach (Array.transpose(({Layer()->available_modes(),
-**!                               Layer()->descriptions()})),
-**!             [string mode,string desc])
-**!    {
-**!	  if ((&lt;"add","equal","replace","replace_hsv","darken",
-**!	        "dissolve","screen","logic_equal">)[mode])
-**!          write(mktag("tr",0,mktag("td",0,"\240")));
-**!
-**! 	  ({lc2,lc2b,li2,li2b,lr2,lzo1})->set_mode(mode);
-**!
-**! 	  object r=
-**! 	     lay( ({ lay(({lca1,lc2}),0,0,80,80),
-**! 		     lay(({lc1,lc2b}),0,0,80,80)->set_offset(80,0),
-**! 		     lay(({li1,li2}),0,0,80,80)->set_offset(160,0),
-**! 		     lay(({li1,li2b}),0,0,80,80)->set_offset(240,0),
-**! 		     lay(({lr1,lr2}),0,0,80,80)->set_offset(320,0),
-**! 		     lay(({lzo0,lzo1}),400,0,160,80) }),
-**! 		  0,0,560,80);
-**       xv(r);
-**!
-**! 	  write_image(mode,mode,lay(({ltrans,r}))->image(),desc);
-**!    }
-**!
-**!    write(end_tag());
-**!    return 0;
-**! }
-**!
-**! </execute>
-**! </add_appendix>
 */
 
 #include "global.h"
 
 #include <math.h> /* floor */
 
-RCSID("$Id: layers.c,v 1.64 2004/02/03 20:45:25 grubba Exp $");
+RCSID("$Id: layers.c,v 1.98 2006/03/08 01:21:35 peter Exp $");
 
 #include "image_machine.h"
 
 #include "stralloc.h"
 #include "pike_macros.h"
 #include "object.h"
-#include "constants.h"
 #include "interpret.h"
 #include "svalue.h"
 #include "array.h"
@@ -233,16 +31,17 @@ RCSID("$Id: layers.c,v 1.64 2004/02/03 20:45:25 grubba Exp $");
 #include "dmalloc.h"
 #include "operators.h"
 #include "module_support.h"
-#include "opcodes.h"
 
 #include "image.h"
 
-/* This must be included last! */
-#include "module_magic.h"
-
 #ifdef TRY_USE_MMX
+#ifdef HAVE_MMX_H
 #include <mmx.h>
+#else
+#include <asm/mmx.h>
 #endif
+#endif
+
 
 extern struct program *image_program;
 extern struct program *image_layer_program;
@@ -737,7 +536,7 @@ static INLINE void hsv_to_rgb(double h,double s,double v,rgb_group *colorp)
       case 3: 	colorp->r = p;	colorp->g = q;	colorp->b = V;	 break;
       case 4: 	colorp->r = t;	colorp->g = p;	colorp->b = V;	 break;
       case 5: 	colorp->r = V;	colorp->g = p;	colorp->b = q;	 break;
-      default: fatal("unhandled case\n");
+      default: Pike_fatal("unhandled case\n");
    }
 #undef V
 #undef i
@@ -775,9 +574,9 @@ static INLINE void hls_to_rgb(double h,double l,double s,rgb_group *rgb)
    if (s == 0)
    {
 /*  achromatic case  */
-      rgb->r = l;
-      rgb->g = l;
-      rgb->b = l;
+      rgb->r = (COLORTYPE)l;
+      rgb->g = (COLORTYPE)l;
+      rgb->b = (COLORTYPE)l;
    }
    else
    {
@@ -1051,7 +850,7 @@ static void image_layer_alpha(INT32 args)
 
 /*
 **! method object set_alpha_value(float value)
-**! method double alpha_value()
+**! method float alpha_value()
 **!	Set/get the general alpha value of this layer.
 **!	This is a float value between 0 and 1,
 **!	and is multiplied with the alpha channel.
@@ -1080,39 +879,186 @@ static void image_layer_alpha_value(INT32 args)
 **! method array(string) available_modes()
 **!	Set/get layer mode. Mode is one of these:
 **!
-**!  	"normal",
-**!  	"add",
-**!  	"subtract",
-**!  	"multiply",
-**!  	"divide",
-**!  	"modulo",
-**!  	"invsubtract",
-**!  	"invdivide",
-**!  	"invmodulo",
-**!  	"difference",
-**!  	"max",
-**!  	"min",
-**!  	"bitwise_and",
-**!  	"bitwise_or",
-**!  	"bitwise_xor",
+**! <dl compact>
+**! <dt><i>The variables in the expression:</i></dt>
+**! <dt>L</dt><dd><i>The active layer</i></dd>
+**! <dt>S</dt><dd><i>The source layer (the sum of the layers below)</i></dd>
+**! <dt>D</dt><dd><i>The destintion layer (the result)</i></dd>
+**! <dt>Xrgb</dt><dd><i>Layer red (<b>Xr</b>), green (<b>Xg</b>) or blue channel (<b>Xb</b>) </i></dd>
+**! <dt>Xhsv</dt><dd><i>Layer hue (<b>Xh</b>), saturation (<b>Xs</b>) or value channel (<b>Xv</b>) (virtual channels)</i></dd>
+**! <dt>Xhls</dt><dd><i>Layer hue (<b>Xh</b>), lightness channel (<b>Xl</b>) or saturation (<b>Xs</b>) (virtual channels)</i></dd>
+**! <dt>aX</dt><dd><i>Layer alpha, channel in layer alpha</i></dd>
+**! </dl>
+**! <i>All channels are calculated separately, if nothing else is specified.</i>
+**! <execute>
+**! import Image;
 **!
-**!  	"replace",
-**!  	"red",
-**!  	"green",
-**!  	"blue",
+**! void write_image(string desc,
+**! 		     string filename,
+**! 		     Image img,
+**!                  string longdesc)
+**! {
+**!    longdesc = replace(longdesc, ([ "&lt;":"&amp;lt;", "&gt;":"&amp;gt;", "&amp;":"&amp;amp;" ]));
+**!    write(begin_tag("tr"));
+**!    write(mktag("td",(["align":"left","colspan":"2"]),
+**!          mktag("b",0,desc)));
+**!    write(end_tag());
+**!    write(begin_tag("tr"));
+**!    write(mktag("td",(["align":"right"]),illustration_jpeg(img,(["dpi":150.0,"quality":90]))));
+**!    write(mktag("td",(["align":"left","valign":"center"]),longdesc));
+**           (replace(longdesc,({",",";",")"}),
+**                    ({",<wbr>",";<wbr>",")<wbr>"}))/
+**            "<wbr>")/1*({mktag("wbr")}) ) );
+**!    write(end_tag()+"\n");
+**! }
 **!
-**!  	"replace_hsv",
-**!  	"hue",
-**!  	"saturation",
-**!  	"value",
-**!  	"color",
+**! int main()
+**! {
+**!    object ltrans=Layer((["image":
+**! 			     Image(32,32,160,160,160)->
+**! 			     box(0,0,15,15,96,96,96)->
+**! 			     box(16,16,31,31,96,96,96)->scale(0.5),
+**! 			     "tiled":1,
+**! 			     "mode":"normal"]));
 **!
-**!  	"darken",
-**!  	"lighten",
+**!    object circle=load(fix_image_path("circle50.pnm"));
+**!    object image_test=load(fix_image_path("image_ill.pnm"));
 **!
-**!  	"dissolve",
-**!  	"behind",
-**!  	"erase",
+**!    object lc1=
+**! 	     Layer((["image":circle->clear(255,0,0),
+**! 		     "alpha":circle,
+**! 		     "xoffset":5,
+**! 		     "yoffset":5]));
+**!
+**!    object lc2=
+**! 	  Layer((["image":circle->clear(0,0,255),
+**! 		  "alpha":circle,
+**! 		  "xoffset":25,
+**! 		  "yoffset":25]));
+**!    object lc2b=
+**! 	  Layer((["image":circle,
+**! 		  "alpha":circle*({0,0,255}),
+**! 		  "xoffset":25,
+**! 		  "yoffset":25]));
+**!
+**!    object li1=
+**! 	  Layer((["image":image_test->scale(80,0),
+**! 		  "mode":"normal",
+**! 		  "xoffset":0,
+**! 		  "yoffset":0]));
+**!
+**!    object li2=
+**! 	  lay(
+**! 	     ({
+**! 	     (["image":circle->clear(255,0,0),"alpha":circle,
+**! 	       "xoffset":5,"yoffset":0,"mode":"add"]),
+**! 	     (["image":circle->clear(0,255,0),"alpha":circle,
+**! 	       "xoffset":25,"yoffset":0,"mode":"add"]),
+**! 	     (["image":circle->clear(0,0,255),"alpha":circle,
+**! 	       "xoffset":15,"yoffset":20,"mode":"add"]),
+**! 	     (["image":circle->clear(0,0,0)->scale(0.5),
+**! 	       "alpha":circle->scale(0.5),
+**! 	       "xoffset":0,"yoffset":55]),
+**! 	     (["image":circle->clear(255,255,255)->scale(0.5),
+**! 	       "alpha":circle->scale(0.5),
+**! 	       "xoffset":55,"yoffset":55])
+**! 	     }));
+**!
+**!    Layer li2b=Layer(li2->image()->clear(255,255,255),li2->image());
+**!
+**!    object lzo0=
+**! 	  lay( ({
+**! 	     Layer(Image(4,10)
+**! 		   ->tuned_box(0,0,3,8,({({255,255,255}),({255,255,255}),
+**! 					 ({0,0,0}),({0,0,0})}))
+**! 		   ->scale(40,80))
+**! 	     ->set_offset(0,0),
+**! 	     Layer(Image(40,80)
+**! 		   ->tuned_box(0,0,40,78,({({255,255,255}),({255,255,255}),
+**! 					  ({0,0,0}),({0,0,0})})))
+**! 	     ->set_offset(40,0),
+**! 	     Layer(Image(80,80)
+**! 		   ->tuned_box(0,0,80,80,({({255,0,0}),({255,255,0}),
+**! 					  ({0,0,255}),({0,255,0})})))
+**! 	     ->set_offset(80,0),
+**! 	  }) );
+**!
+**!    object scale=
+**! 	  Image(4,80)
+**! 	  ->tuned_box(0,0,40,78,({({255,255,255}),({255,255,255}),
+**! 				  ({0,0,0}),({0,0,0})}));
+**!    object lzo1=
+**! 	  lay( ({
+**! 	     Layer(scale)->set_offset(2,0),
+**! 	     Layer(scale->invert())->set_offset(6,0),
+**! 	     Layer(Image(26,80)->test())->set_offset(12,0),
+**! 	     Layer(scale)->set_offset(42,0),
+**! 	     Layer(scale->invert())->set_offset(46,0),
+**! 	     Layer(Image(26,80)->test())->set_offset(52,0),
+**! 	     Layer(scale)->set_offset(82,0),
+**! 	     Layer(scale->invert())->set_offset(86,0),
+**! 	     Layer(Image(26,80)->test())->set_offset(92,0),
+**! 	     Layer(scale)->set_offset(122,0),
+**! 	     Layer(scale->invert())->set_offset(126,0),
+**! 	     Layer(Image(26,80,"white"),
+**! 		   Image(26,80)->test())->set_offset(132,0),
+**! 	  }));
+**!
+**!    object lca1;
+**!
+**!    object a=
+**! 	  lay( ({ lca1=lay(({Layer((["fill":"white"])),
+**!                          lc1}),0,0,80,80),
+**! 		  lay(({lc1}),0,0,80,80)->set_offset(80,0),
+**! 		  lay(({li1}),0,0,80,80)->set_offset(160,0),
+**! 		  lay(({li1}),0,0,80,80)->set_offset(240,0),
+**! 		  lzo0->set_offset(320,0)}),
+**! 	       0,0,480,80);
+**!
+**!    object b=
+**! 	  lay( ({ lay(({lc2}),0,0,80,80),
+**! 		  lay(({lc2b}),0,0,80,80)->set_offset(80,0),
+**! 		  lay(({li2}),0,0,80,80)->set_offset(160,0),
+**! 		  lay(({li2b}),0,0,80,80)->set_offset(240,0),
+**! 		  lzo1->set_offset(320,0)}),
+**! 	       0,0,480,80);
+**!
+**    xv(a); xv(b);
+**!
+**!    write(begin_tag("table",(["cellspacing":"0","cellpadding":"1"])));
+**!
+**!    write_image("top layer","b",lay(({ltrans,b}))->image(),
+**!		   "");
+**!    write_image("bottom layer","a",lay(({ltrans,a}))->image(),
+**!                "");
+**!
+**!    foreach (Array.transpose(({Layer()->available_modes(),
+**!                               Layer()->descriptions()})),
+**!             [string mode,string desc])
+**!    {
+**!	  if ((&lt;"add","equal","replace","replace_hsv","darken",
+**!	        "dissolve","screen","logic_equal">)[mode])
+**!          write(mktag("tr",0,mktag("td",0,"\240")));
+**!
+**! 	  ({lc2,lc2b,li2,li2b,lzo1})->set_mode(mode);
+**!
+**! 	  object r=
+**! 	     lay( ({ lay(({lca1,lc2}),0,0,80,80),
+**! 		     lay(({lc1,lc2b}),0,0,80,80)->set_offset(80,0),
+**! 		     lay(({li1,li2}),0,0,80,80)->set_offset(160,0),
+**! 		     lay(({li1,li2b}),0,0,80,80)->set_offset(240,0),
+**! 		     lay(({lzo0,lzo1}),320,0,160,80) }),
+**! 		  0,0,480,80);
+**       xv(r);
+**!
+**! 	  write_image(mode,mode,lay(({ltrans,r}))->image(),desc);
+**!    }
+**!
+**!    write(end_tag());
+**!    return 0;
+**! }
+**!
+**! </execute>
 **!
 **!	<ref>available_modes</ref>() simply gives an array
 **!	containing the names of these modes.
@@ -1157,7 +1103,7 @@ static void image_layer_mode(INT32 args)
 	 return;
       }
 
-   fatal("illegal mode: %p\n", (void *)layer_mode[i].func);
+   Pike_fatal("illegal mode: %p\n", (void *)layer_mode[i-1].func);
 }
 
 static void image_layer_available_modes(INT32 args)
@@ -1171,13 +1117,17 @@ static void image_layer_available_modes(INT32 args)
    f_aggregate(LAYER_MODES);
 }
 
+/*
+**! method array(string) description()
+**!     Layer descriptions
+*/
 static void image_layer_descriptions(INT32 args)
 {
    int i;
    pop_n_elems(args);
 
    for (i=0; i<LAYER_MODES; i++)
-     push_string(make_shared_string(layer_mode[i].desc));
+     push_text(layer_mode[i].desc);
 
    f_aggregate(LAYER_MODES);
 }
@@ -1264,7 +1214,7 @@ static void image_layer_fill_alpha(INT32 args)
 
 static void image_layer_set_offset(INT32 args)
 {
-   get_all_args("Image.Layer->set_offset",args,"%i%i",
+   get_all_args("Image.Layer->set_offset",args,"%d%d", /* INT32! */
 		&(THIS->xoffs),&(THIS->yoffs));
    pop_n_elems(args);
    ref_push_object(THISOBJ);
@@ -1304,9 +1254,9 @@ static void image_layer_ysize(INT32 args)
 
 static void image_layer_set_tiled(INT32 args)
 {
-   get_all_args("Image.Layer->set_offset",args,"%i",
-		&(THIS->tiled));
-   THIS->tiled=!!THIS->tiled;
+   INT_TYPE tiled;
+   get_all_args("Image.Layer->set_offset",args,"%i",&tiled);
+   THIS->tiled=!!tiled;
    THIS->really_optimize_alpha=really_optimize_p(THIS);
    pop_n_elems(args);
    ref_push_object(THISOBJ);
@@ -1427,7 +1377,8 @@ static void image_layer_create(INT32 args)
    {
       rgb_group col=black,alpha=white;
 
-      get_all_args("Image.Layer",args,"%i%i",&(THIS->xsize),&(THIS->ysize));
+      get_all_args("Image.Layer",args,"%d%d", /* watch the type: INT32 */
+		   &(THIS->xsize),&(THIS->ysize));
       if (args>2)
 	 if (!image_color_arg(2-args,&col))
 	    SIMPLE_BAD_ARG_ERROR("Image.Layer",3,"Image.Color");
@@ -1471,6 +1422,20 @@ static void image_layer_create(INT32 args)
 
 /*** layer object *****************************************/
 
+/*
+**! method mapping(string:mixed)|string cast()
+**! ([ "xsize":int,
+**!    "ysize":int,
+**!    "image":image,
+**!    "alpha":image,
+**!    "xoffset":int,
+**!    "yoffset":int,
+**!    "fill":image,
+**!    "fill_alpha":image
+**!    "tiled":int,
+**!    "mode":string
+**! ])
+ */
 static void image_layer_cast(INT32 args)
 {
    if (!args)
@@ -1482,25 +1447,65 @@ static void image_layer_cast(INT32 args)
 	 int n=0;
 	 pop_n_elems(args);
 
-	 push_text("xsize");        push_int(THIS->xsize);         n++;
-	 push_text("ysize");        push_int(THIS->ysize);         n++;
-	 push_text("image");        image_layer_image(0);          n++;
-	 push_text("alpha");        image_layer_alpha(0);          n++;
-	 push_text("xoffset");      push_int(THIS->xoffs);         n++;
-	 push_text("yoffset");      push_int(THIS->yoffs);         n++;
-	 push_text("alpha_value");  push_float(THIS->alpha_value); n++;
-	 push_text("fill");         image_layer_fill(0);           n++;
-	 push_text("fill_alpha");   image_layer_fill_alpha(0);     n++;
-	 push_text("tiled");        push_int(THIS->tiled);         n++;
-	 push_text("mode");         image_layer_mode(0);           n++;
+	 push_constant_text("xsize");       push_int(THIS->xsize);         n++;
+	 push_constant_text("ysize");       push_int(THIS->ysize);         n++;
+	 push_constant_text("image");       image_layer_image(0);          n++;
+	 push_constant_text("alpha");       image_layer_alpha(0);          n++;
+	 push_constant_text("xoffset");     push_int(THIS->xoffs);         n++;
+	 push_constant_text("yoffset");     push_int(THIS->yoffs);         n++;
+	 push_constant_text("alpha_value"); push_float(THIS->alpha_value); n++;
+	 push_constant_text("fill");        image_layer_fill(0);           n++;
+	 push_constant_text("fill_alpha");  image_layer_fill_alpha(0);     n++;
+	 push_constant_text("tiled");       push_int(THIS->tiled);         n++;
+	 push_constant_text("mode");        image_layer_mode(0);           n++;
 
 	 f_aggregate_mapping(n*2);
 
 	 return;
       }
+      else if (strncmp(Pike_sp[-args].u.string->str,"string",6)==0)
+      {
+	size_t size = THIS->xsize*THIS->ysize, i;
+	struct pike_string *s = begin_shared_string(size*4);
+	rgb_group *img = 0;
+	rgb_group *alp = 0;
+
+	pop_n_elems(args);
+	if(THIS->img)
+	  img = THIS->img->img;
+	if(THIS->alp)
+	  alp = THIS->alp->img;
+
+	if(img && alp)
+	  for(i=0; i<size; i++) {
+	    s->str[i*4+0] = img[i].r;
+	    s->str[i*4+1] = img[i].g;
+	    s->str[i*4+2] = img[i].b;
+	    s->str[i*4+3] = alp[i].r;
+	  }
+	else if(img)
+	  for(i=0; i<size; i++) {
+	    s->str[i*4+0] = img[i].r;
+	    s->str[i*4+1] = img[i].g;
+	    s->str[i*4+2] = img[i].b;
+	    s->str[i*4+3] = 255;
+	  }
+	else if(alp)
+	  for(i=0; i<size; i++) {
+	    s->str[i*4+0] = 255;
+	    s->str[i*4+1] = 255;
+	    s->str[i*4+2] = 255;
+	    s->str[i*4+3] = alp[i].r;
+	  }
+	else
+	  memset(s->str, 0, size*4);
+
+	push_string(end_shared_string(s));
+	return;
+      }
    }
    SIMPLE_BAD_ARG_ERROR("Image.Colortable->cast",1,
-			"string(\"mapping\"|\"array\"|\"string\")");
+			"string(\"mapping\"|\"string\")");
 
 }
 
@@ -1583,9 +1588,9 @@ static void lm_normal(rgb_group *s,rgb_group *l,rgb_group *d,
 
 
 #if defined(__ECL) && 0
-#define WARN_TRACE(X)	static char PIKE_CONCAT(foo__, X) (double d) { return (char)d; }
+#define WARN_TRACE(X)	static char PIKE_CONCAT(foo__, X) (double d) { return (char)d; };
 #else /* !__ECL */
-#define WARN_TRACE(X)
+#define WARN_TRACE(X) 
 #endif /* __ECL */
 
 /* operators from template */
@@ -1596,7 +1601,7 @@ static void lm_normal(rgb_group *s,rgb_group *l,rgb_group *d,
 #define L_TRUNC(X) MINIMUM(255,(X))
 #define L_OPER(A,B) ((A)+(int)(B))
 #define L_MMX_OPER(A,MMXR) paddusb_m2r(A,MMXR)
-WARN_TRACE(1);
+WARN_TRACE(1)
 #include "layer_oper.h"
 #undef L_MMX_OPER
 #undef LM_FUNC
@@ -1607,7 +1612,7 @@ WARN_TRACE(1);
 #define L_TRUNC(X) MINIMUM(255,(X))
 #define L_OPER(A,B) ((A)+(int)(B))
 #define L_MMX_OPER(A,MMXR) paddusb_m2r(A,MMXR)
-WARN_TRACE(1);
+WARN_TRACE(1)
 #include "layer_oper.h"
 #undef L_MMX_OPER
 #undef LM_FUNC
@@ -1618,7 +1623,7 @@ WARN_TRACE(1);
 #define L_TRUNC(X) MAXIMUM(0,(X))
 #define L_OPER(A,B) ((A)-(int)(B))
 #define L_MMX_OPER(A,MMXR) psubusb_m2r(A,MMXR)
-WARN_TRACE(2);
+WARN_TRACE(2)
 #include "layer_oper.h"
 #undef L_MMX_OPER
 #undef LM_FUNC
@@ -1628,7 +1633,7 @@ WARN_TRACE(2);
 #define LM_FUNC lm_multiply
 #define L_TRUNC(X) (X)
 #define L_OPER(A,B) CCUT((A)*(int)(B))
-WARN_TRACE(3);
+WARN_TRACE(3)
 #include "layer_oper.h"
 #undef LM_FUNC
 #undef L_TRUNC
@@ -1637,7 +1642,7 @@ WARN_TRACE(3);
 #define LM_FUNC lm_divide
 #define L_TRUNC(X) MINIMUM(255,(X))
 #define L_OPER(A,B) MINIMUM( DOUBLE_TO_INT((A)/C2F(1+(int)(B))), COLORMAX)
-WARN_TRACE(4);
+WARN_TRACE(4)
 #include "layer_oper.h"
 #undef LM_FUNC
 #undef L_TRUNC
@@ -1646,7 +1651,7 @@ WARN_TRACE(4);
 #define LM_FUNC lm_negdivide
 #define L_TRUNC(X) MINIMUM(255,(X))
 #define L_OPER(A,B) 1.0-MINIMUM( DOUBLE_TO_INT((A)/C2F(1+(int)(B))), COLORMAX)
-WARN_TRACE(4);
+WARN_TRACE(4)
 #include "layer_oper.h"
 #undef LM_FUNC
 #undef L_TRUNC
@@ -1655,7 +1660,7 @@ WARN_TRACE(4);
 #define LM_FUNC lm_modulo
 #define L_TRUNC(X) (DOUBLE_TO_COLORTYPE(X))
 #define L_OPER(A,B) ((A)%((B)?(B):1))
-WARN_TRACE(5);
+WARN_TRACE(5)
 #include "layer_oper.h"
 #undef LM_FUNC
 #undef L_TRUNC
@@ -1666,7 +1671,7 @@ WARN_TRACE(5);
 #define LM_FUNC lm_invsubtract
 #define L_TRUNC(X) MAXIMUM(0,(X))
 #define L_OPER(A,B) ((B)-(int)(A))
-WARN_TRACE(6);
+WARN_TRACE(6)
 #include "layer_oper.h"
 #undef LM_FUNC
 #undef L_TRUNC
@@ -1675,7 +1680,7 @@ WARN_TRACE(6);
 #define LM_FUNC lm_invdivide
 #define L_TRUNC(X) MINIMUM(255,(X))
 #define L_OPER(A,B) MINIMUM( DOUBLE_TO_INT((B)/C2F(1+(int)(A))), COLORMAX)
-WARN_TRACE(7);
+WARN_TRACE(7)
 #include "layer_oper.h"
 #undef LM_FUNC
 #undef L_TRUNC
@@ -1684,7 +1689,7 @@ WARN_TRACE(7);
 #define LM_FUNC lm_invmodulo
 #define L_TRUNC(X) (DOUBLE_TO_COLORTYPE(X))
 #define L_OPER(A,B) ((B)%((A)?(A):1))
-WARN_TRACE(8);
+WARN_TRACE(8)
 #include "layer_oper.h"
 #undef LM_FUNC
 #undef L_TRUNC
@@ -1693,7 +1698,7 @@ WARN_TRACE(8);
 #define LM_FUNC lm_idivide
 #define L_TRUNC(X) MINIMUM(255,(X))
 #define L_OPER(A,B) MINIMUM( DOUBLE_TO_INT((A)/C2F(COLORMAX+1-(int)(B))), COLORMAX)
-WARN_TRACE(4);
+WARN_TRACE(4)
 #include "layer_oper.h"
 #undef LM_FUNC
 #undef L_TRUNC
@@ -1702,7 +1707,7 @@ WARN_TRACE(4);
 #define LM_FUNC lm_imultiply
 #define L_TRUNC(X) (X)
 #define L_OPER(A,B) CCUT((A)*(COLORMAX-(int)(B)))
-WARN_TRACE(3);
+WARN_TRACE(3)
 #include "layer_oper.h"
 #undef LM_FUNC
 #undef L_TRUNC
@@ -1711,7 +1716,7 @@ WARN_TRACE(3);
 #define LM_FUNC lm_invidivide
 #define L_TRUNC(X) MINIMUM(255,(X))
 #define L_OPER(A,B) MINIMUM( DOUBLE_TO_INT((B)/C2F(COLORMAX+1-(int)(A))), COLORMAX)
-WARN_TRACE(7);
+WARN_TRACE(7)
 #include "layer_oper.h"
 #undef LM_FUNC
 #undef L_TRUNC
@@ -1722,7 +1727,7 @@ WARN_TRACE(7);
 #define LM_FUNC lm_difference
 #define L_TRUNC(X) (DOUBLE_TO_COLORTYPE(X))
 #define L_OPER(A,B) abs((A)-(B))
-WARN_TRACE(9);
+WARN_TRACE(9)
 #include "layer_oper.h"
 #undef LM_FUNC
 #undef L_TRUNC
@@ -1731,7 +1736,7 @@ WARN_TRACE(9);
 #define LM_FUNC lm_max
 #define L_TRUNC(X) (DOUBLE_TO_COLORTYPE(X))
 #define L_OPER(A,B) MAXIMUM((A),(B))
-WARN_TRACE(10);
+WARN_TRACE(10)
 #include "layer_oper.h"
 #undef LM_FUNC
 #undef L_TRUNC
@@ -1740,7 +1745,7 @@ WARN_TRACE(10);
 #define LM_FUNC lm_min
 #define L_TRUNC(X) (DOUBLE_TO_COLORTYPE(X))
 #define L_OPER(A,B) MINIMUM((A),(B))
-WARN_TRACE(11);
+WARN_TRACE(11)
 #include "layer_oper.h"
 #undef LM_FUNC
 #undef L_TRUNC
@@ -1749,7 +1754,7 @@ WARN_TRACE(11);
 #define LM_FUNC lm_bitwise_and
 #define L_TRUNC(X) (DOUBLE_TO_COLORTYPE(X))
 #define L_OPER(A,B) ((A)&(B))
-WARN_TRACE(12);
+WARN_TRACE(12)
 #include "layer_oper.h"
 #undef LM_FUNC
 #undef L_TRUNC
@@ -1758,7 +1763,7 @@ WARN_TRACE(12);
 #define LM_FUNC lm_bitwise_or
 #define L_TRUNC(X) (DOUBLE_TO_COLORTYPE(X))
 #define L_OPER(A,B) ((A)|(B))
-WARN_TRACE(13);
+WARN_TRACE(13)
 #include "layer_oper.h"
 #undef LM_FUNC
 #undef L_TRUNC
@@ -1767,7 +1772,7 @@ WARN_TRACE(13);
 #define LM_FUNC lm_bitwise_xor
 #define L_TRUNC(X) (DOUBLE_TO_COLORTYPE(X))
 #define L_OPER(A,B) ((A)^(B))
-WARN_TRACE(14);
+WARN_TRACE(14)
 #include "layer_oper.h"
 #undef LM_FUNC
 #undef L_TRUNC
@@ -1778,7 +1783,7 @@ WARN_TRACE(14);
 #define LM_FUNC lm_equal
 #define L_TRUNC(X) (DOUBLE_TO_COLORTYPE(X))
 #define L_OPER(A,B) (((A)==(B))?COLORMAX:0)
-WARN_TRACE(15);
+WARN_TRACE(15)
 #include "layer_oper.h"
 #undef LM_FUNC
 #undef L_TRUNC
@@ -1787,7 +1792,7 @@ WARN_TRACE(15);
 #define LM_FUNC lm_not_equal
 #define L_TRUNC(X) (DOUBLE_TO_COLORTYPE(X))
 #define L_OPER(A,B) (((A)!=(B))?COLORMAX:0)
-WARN_TRACE(16);
+WARN_TRACE(16)
 #include "layer_oper.h"
 #undef LM_FUNC
 #undef L_TRUNC
@@ -1796,7 +1801,7 @@ WARN_TRACE(16);
 #define LM_FUNC lm_less
 #define L_TRUNC(X) (DOUBLE_TO_COLORTYPE(X))
 #define L_OPER(A,B) (((A)>(B))?COLORMAX:0)
-WARN_TRACE(17);
+WARN_TRACE(17)
 #include "layer_oper.h"
 #undef LM_FUNC
 #undef L_TRUNC
@@ -1805,7 +1810,7 @@ WARN_TRACE(17);
 #define LM_FUNC lm_more
 #define L_TRUNC(X) (DOUBLE_TO_COLORTYPE(X))
 #define L_OPER(A,B) (((A)<(B))?COLORMAX:0)
-WARN_TRACE(18);
+WARN_TRACE(18)
 #include "layer_oper.h"
 #undef LM_FUNC
 #undef L_TRUNC
@@ -1814,7 +1819,7 @@ WARN_TRACE(18);
 #define LM_FUNC lm_less_or_equal
 #define L_TRUNC(X) (DOUBLE_TO_COLORTYPE(X))
 #define L_OPER(A,B) (((A)>=(B))?COLORMAX:0)
-WARN_TRACE(19);
+WARN_TRACE(19)
 #include "layer_oper.h"
 #undef LM_FUNC
 #undef L_TRUNC
@@ -1823,7 +1828,7 @@ WARN_TRACE(19);
 #define LM_FUNC lm_more_or_equal
 #define L_TRUNC(X) (DOUBLE_TO_COLORTYPE(X))
 #define L_OPER(A,B) (((A)<=(B))?COLORMAX:0)
-WARN_TRACE(20);
+WARN_TRACE(20)
 #include "layer_oper.h"
 #undef LM_FUNC
 #undef L_TRUNC
@@ -1835,7 +1840,7 @@ WARN_TRACE(20);
 #define LM_FUNC lm_logic_equal
 #define L_OPER(A,B) ((A)==(B))
 #define L_TRANS white
-WARN_TRACE(21);
+WARN_TRACE(21)
 #include "layer_oper.h"
 #undef L_TRANS
 #undef L_OPER
@@ -1846,7 +1851,7 @@ WARN_TRACE(21);
 #define LM_FUNC lm_logic_not_equal
 #define L_OPER(A,B) ((A)!=(B))
 #define L_TRANS black
-WARN_TRACE(22);
+WARN_TRACE(22)
 #include "layer_oper.h"
 #undef L_TRANS
 #undef L_OPER
@@ -1857,7 +1862,7 @@ WARN_TRACE(22);
 #define LM_FUNC lm_logic_strict_less
 #define L_OPER(A,B) ((A)>(B))
 #define L_TRANS white
-WARN_TRACE(23);
+WARN_TRACE(23)
 #include "layer_oper.h"
 #undef L_TRANS
 #undef L_OPER
@@ -1868,7 +1873,7 @@ WARN_TRACE(23);
 #define LM_FUNC lm_logic_strict_more
 #define L_OPER(A,B) ((A)<(B))
 #define L_TRANS white
-WARN_TRACE(24);
+WARN_TRACE(24)
 #include "layer_oper.h"
 #undef L_TRANS
 #undef L_OPER
@@ -1879,7 +1884,7 @@ WARN_TRACE(24);
 #define LM_FUNC lm_logic_strict_less_or_equal
 #define L_OPER(A,B) ((A)>=(B))
 #define L_TRANS white
-WARN_TRACE(25);
+WARN_TRACE(25)
 #include "layer_oper.h"
 #undef L_TRANS
 #undef L_OPER
@@ -1890,7 +1895,7 @@ WARN_TRACE(25);
 #define LM_FUNC lm_logic_strict_more_or_equal
 #define L_OPER(A,B) ((A)<=(B))
 #define L_TRANS white
-WARN_TRACE(26);
+WARN_TRACE(26)
 #include "layer_oper.h"
 #undef L_TRANS
 #undef L_OPER
@@ -2407,7 +2412,6 @@ static void lm_spec_burn_alpha(struct layer *ly,
 	     ly->fill.g!=0 ||
 	     ly->fill.b!=0)
 	 {
-	    rgb_group fill=ly->fill;
 	    while (len--)
 	    {
 	       d->r=MINIMUM(s->r+la->r,COLORMAX);
@@ -2593,6 +2597,7 @@ static INLINE void img_lay_stroke(struct layer *ly,
 				  int len)
 {
    if (len<0) Pike_error("internal error: stroke len < 0\n");
+   if (!ly->row_func) Pike_error("internal error: row_func=NULL\n");
 
    if (ly->row_func==(lm_row_func*)lm_spec_burn_alpha)
    {
@@ -2634,8 +2639,6 @@ static INLINE void img_lay_stroke(struct layer *ly,
    }
    else
    {
-      int i;
-
 /* fprintf(stderr,"ly=%p len=%d\n",ly,len); */
 
       while (len>SNUMPIXS)
@@ -2660,6 +2663,9 @@ static INLINE void img_lay_line(struct layer *ly,
 				int y, /* y in ly layer */
 				rgb_group *d,rgb_group *da)
 {
+/*     fprintf(stderr,"tiled:%d xoffs:%d xsize:%d y:%d\n", */
+/*  	   ly->tiled,xoffs,xsize,y); */
+
    if (!ly->tiled)
    {
       int len;
@@ -2723,6 +2729,9 @@ static INLINE void img_lay_line(struct layer *ly,
 	 int len;
 	 if (xoffs<0) xoffs+=ly->xsize;
 	 len=ly->xsize-xoffs;
+	 if (len>xsize) len=xsize;
+
+/*  	 fprintf(stderr,"a xoffs=%d len=%d xsize=%d ly->xsize:%d\n",xoffs,len,xsize,ly->xsize); */
 
 	 img_lay_stroke(ly,l?l+xoffs:NULL,
 			la?la+(xoffs%ly->xsize):NULL,
@@ -2735,6 +2744,8 @@ static INLINE void img_lay_line(struct layer *ly,
       }
       while (xsize>ly->xsize)
       {
+/*  	 fprintf(stderr,"b xsize=%d\n",xsize); */
+
 	 img_lay_stroke(ly,l,la,s,sa,d,da,ly->xsize);
 	 da+=ly->xsize;
 	 d+=ly->xsize;
@@ -2743,7 +2754,10 @@ static INLINE void img_lay_line(struct layer *ly,
 	 xsize-=ly->xsize;
       }
       if (xsize)
+      {
+/*  	 fprintf(stderr,"c xsize=%d\n",xsize); */
 	 img_lay_stroke(ly,l,la,s,sa,d,da,xsize);
+      }
    }
 }
 
@@ -2819,8 +2833,6 @@ void img_lay(struct layer **layer,
 		(layer[z]->yoffs<=y+dest->yoffs &&
 		 y+dest->yoffs<layer[z]->yoffs+layer[z]->ysize))
 	    {
-	       rgb_group *tmp;
-
 /* 	       if (!layer[z]->really_optimize_alpha) */
 /* 		  fprintf(stderr,"huh %d\n",z); */
 /* 	       if (!(layer[z]->yoffs>=y+dest->yoffs &&  */
@@ -2884,8 +2896,8 @@ void img_lay(struct layer **layer,
 
 /*
 **! module Image
-**! method Image.Layer lay(array(Image.Layer|mapping))
-**! method Image.Layer lay(array(Image.Layer|mapping),int xoffset,int yoffset,int xsize,int ysize)
+**! method Image.Layer lay(array(Image.Layer|mapping) layers)
+**! method Image.Layer lay(array(Image.Layer|mapping) layers,int xoffset,int yoffset,int xsize,int ysize)
 **!	Combine layers.
 **! returns a new layer object.
 **!
@@ -3018,13 +3030,13 @@ void image_lay(INT32 args)
 
 /**  image-object operations  *************************/
 
-static INLINE struct layer *push_new_layer()
+static INLINE struct layer *push_new_layer(void)
 {
    push_object(clone_object(image_layer_program,0));
    return (struct layer*)get_storage(Pike_sp[-1].u.object,image_layer_program);
 }
 
-static INLINE struct layer *clone_this_layer()
+static INLINE struct layer *clone_this_layer(void)
 {
    struct layer *l;
    l=push_new_layer();
@@ -3088,7 +3100,7 @@ static void image_layer_crop(INT32 args)
    int zot=0;
    struct image *img = NULL;
 
-   get_all_args("Image.Layer->crop",args,"%d%d%d%d",&x,&y,&xz,&yz);
+   get_all_args("Image.Layer->crop",args,"%i%i%i%i",&x,&y,&xz,&yz);
 
    l=clone_this_layer();
    if (x<=l->xoffs) x=l->xoffs; else zot++;
@@ -3119,8 +3131,9 @@ static void image_layer_crop(INT32 args)
 	 Pike_error("No image returned from image->copy\n");
       if (img->xsize!=xz || img->ysize!=yz)
 	 Pike_error("Image returned from image->copy had "
-	       "unexpected size (%d,%d, expected %d,%d)\n",
-		img->xsize,img->ysize,(INT32)xz,(INT32)yz);
+	       "unexpected size (%"PRINTPIKEINT"d,%"PRINTPIKEINT"d,"
+		    " expected %"PRINTPIKEINT"d,%"PRINTPIKEINT"d)\n",
+		img->xsize,img->ysize,xz,yz);
 
       free_object(l->image);
       l->image=Pike_sp[-1].u.object;
@@ -3147,8 +3160,9 @@ static void image_layer_crop(INT32 args)
 	 Pike_error("No image returned from alpha->copy\n");
       if (img->xsize!=xz || img->ysize!=yz)
 	 Pike_error("Image returned from alpha->copy had "
-	       "unexpected size (%d,%d, expected %d,%d)\n",
-	       img->xsize,img->ysize,(INT32)xz,(INT32)yz);
+	       "unexpected size (%"PRINTPIKEINT"d,%"PRINTPIKEINT"d, "
+		    "expected %"PRINTPIKEINT"d,%"PRINTPIKEINT"d)\n",
+	       img->xsize,img->ysize,xz,yz);
       free_object(l->alpha);
       l->alpha=Pike_sp[-1].u.object;
       Pike_sp--;
@@ -3194,7 +3208,7 @@ static void image_layer_find_autocrop(INT32 args)
    INT_TYPE l=1,r=1,t=1,b=1;
 
    if (args>3)
-      get_all_args("find_autocrop",args,"%d%d%d%d",&l,&r,&t,&b);
+      get_all_args("find_autocrop",args,"%i%i%i%i",&l,&r,&t,&b);
 
    if (!THIS->tiled) {
       if (THIS->alpha)
@@ -3246,12 +3260,12 @@ static void image_layer__sprintf( INT32 args )
   int x;
   if (args != 2 )
     SIMPLE_TOO_FEW_ARGS_ERROR("_sprintf",2);
-  if (sp[-args].type!=T_INT)
+  if (Pike_sp[-args].type!=T_INT)
     SIMPLE_BAD_ARG_ERROR("_sprintf",0,"integer");
-  if (sp[1-args].type!=T_MAPPING)
+  if (Pike_sp[1-args].type!=T_MAPPING)
     SIMPLE_BAD_ARG_ERROR("_sprintf",1,"mapping");
 
-  x = sp[-2].u.integer;
+  x = Pike_sp[-2].u.integer;
 
   pop_n_elems( 2 );
   switch( x )
@@ -3276,8 +3290,6 @@ static void image_layer__sprintf( INT32 args )
 
 void init_image_layers(void)
 {
-   char buf[100];
-   char buf2[sizeof(INT32)];
    int i;
 
    for (i=0; i<LAYER_MODES; i++)
