@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: acconfig.h,v 1.14 2004/07/15 08:39:09 grubba Exp $
+|| $Id: acconfig.h,v 1.17 2006/08/31 10:10:18 mast Exp $
 */
 
 /*
@@ -35,6 +35,12 @@
 /* Define if you have mysql_options(). */
 #undef HAVE_MYSQL_OPTIONS
 
+/* Define if you have mysql_set_character_set(). */
+#undef HAVE_MYSQL_SET_CHARACTER_SET
+
+/* Define if you have mysql_character_set_name(). */
+#undef HAVE_MYSQL_CHARACTER_SET_NAME
+
 /* Define if you have the mysql_port variable. */
 #undef HAVE_MYSQL_PORT
 
@@ -71,4 +77,10 @@
 /* Define if your mysql.h defines SHUTDOWN_DEFAULT */
 #undef HAVE_SHUTDOWN_DEFAULT
 
+/* Define if your mysql.h defines FIELD_TYPE_GEOMETRY */
+#undef HAVE_FIELD_TYPE_GEOMETRY
+
+/* Define if your mysql.h defines CLIENT_INTERACTIVE */
+#undef HAVE_CLIENT_INTERACTIVE
+
 #endif /* PIKE_MYSQL_CONFIG_H */
