@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: mysql.c,v 1.83 2005/08/09 16:30:22 grubba Exp $
+|| $Id: mysql.c,v 1.91 2006/09/16 18:26:33 mast Exp $
 */
 
 /*
@@ -97,7 +97,7 @@
  * Globals
  */
 
-RCSID("$Id: mysql.c,v 1.83 2005/08/09 16:30:22 grubba Exp $");
+RCSID("$Id: mysql.c,v 1.91 2006/09/16 18:26:33 mast Exp $");
 
 /*! @module Mysql
  *!
@@ -158,11 +158,12 @@ static MUTEX_T stupid_port_lock;
 #define MYSQL_DISALLOW()
 #endif /* _REENTRANT */
 
+#define string_has_null(STR)	((ptrdiff_t)strlen((STR)->str) != (STR)->len)
+
 #define CHECK_8BIT_NONBINARY_STRING(FUNC, ARG) do {			\
     if (sp[ARG-1-args].type != T_STRING ||				\
 	sp[ARG-1-args].u.string->size_shift ||				\
-	((ptrdiff_t) strlen (sp[ARG-1-args].u.string->str) !=		\
-	 sp[ARG-1-args].u.string->len))					\
+        string_has_null(sp[ARG-1-args].u.string))			\
       SIMPLE_BAD_ARG_ERROR (FUNC, ARG, "string (nonbinary 8bit)");	\
   } while (0)
 
@@ -215,6 +216,16 @@ static void exit_mysql_struct(struct object *o)
     free_string(PIKE_MYSQL->host);
     PIKE_MYSQL->host = NULL;
   }
+  if (PIKE_MYSQL->options) {
+    free_mapping (PIKE_MYSQL->options);
+    PIKE_MYSQL->options = NULL;
+  }
+#ifndef HAVE_MYSQL_SET_CHARACTER_SET
+  if (PIKE_MYSQL->conn_charset) {
+    free_string (PIKE_MYSQL->conn_charset);
+    PIKE_MYSQL->conn_charset = NULL;
+  }
+#endif
 
   MYSQL_ALLOW();
 
@@ -350,7 +361,7 @@ static void pike_mysql_set_options(struct mapping *options)
 #endif /* HAVE_MYSQL_OPTIONS */
 }
 
-static void pike_mysql_reconnect(void)
+static void pike_mysql_reconnect (int reconnect)
 {
   MYSQL *mysql = PIKE_MYSQL->mysql;
   MYSQL *socket;
@@ -399,6 +410,12 @@ static void pike_mysql_reconnect(void)
     options = (unsigned int)val->u.integer;
   }
 
+#if !defined (HAVE_MYSQL_SET_CHARACTER_SET) && defined (HAVE_MYSQL_OPTIONS) && defined (HAVE_MYSQL_SET_CHARSET_NAME)
+  if (PIKE_MYSQL->conn_charset)
+    mysql_options (mysql, MYSQL_SET_CHARSET_NAME,
+		   PIKE_MYSQL->conn_charset->str);
+#endif
+
   MYSQL_ALLOW();
 
 #if defined(HAVE_MYSQL_PORT) || defined(HAVE_MYSQL_UNIX_PORT)
@@ -452,9 +469,12 @@ static void pike_mysql_reconnect(void)
   }
   
   if (!(PIKE_MYSQL->socket = socket)) {
-    Pike_error("Mysql.mysql(): Couldn't reconnect to SQL-server\n"
-	  "%s\n",
-	  mysql_error(PIKE_MYSQL->mysql));
+    const char *err;
+    MYSQL_ALLOW();
+    err = mysql_error (mysql);
+    MYSQL_DISALLOW();
+    Pike_error("Mysql.mysql(): Couldn't %s to SQL-server: %s\n",
+	       reconnect ? "reconnect" : "connect", err);
   }
 
   if (socket->net.fd >= 0) {
@@ -539,7 +559,17 @@ static void pike_mysql_reconnect(void)
  *!       Change charset directory.
  *!
  *!     @member string "mysql_charset_name"
- *!       Change charset name.
+ *!       Set connection charset - see @[set_charset] for details. The
+ *!       default is @expr{"latin1"@}. As opposed to @[set_charset],
+ *!       this way of specifying the connection charset doesn't
+ *!       require MySQL 4.1.0.
+ *!
+ *!     @member int "unicode_decode_mode"
+ *!       Enable unicode decode mode for the connection if nonzero. In
+ *!       this mode non-binary string results are automatically
+ *!       converted to (possibly wide) unicode strings. An error is
+ *!       thrown if the server doesn't support this. See
+ *!       @[set_unicode_decode_mode].
  *!
  *!     @member string "ssl_key"
  *!       Path to SSL-key for use in SSL-communication.
@@ -566,12 +596,20 @@ static void pike_mysql_reconnect(void)
  *!   Some options may not be implemented. Unimplemented options are
  *!   silently ignored.
  *!
- *!   To use SSL-connections, set the SSL-parameters correctly. They corespond
+ *! @note
+ *!   To use SSL-connections, set the SSL-parameters correctly. They correspond
  *!   to the parameters given to the mysql-client with the same name so make
  *!   sure that the mysql-client works with SSL and set these parameters to
  *!   the same values and everything should work. If SSL-options are loaded
  *!   from a config-file, one may set the connect_options to include
  *!   CLIENT_SSL.
+ *!
+ *! @note
+ *!   If Pike has been built with an old MySQL client lib then it
+ *!   might not be possible to specify some charsets that the server
+ *!   supports with the @expr{"mysql_charset_name"@} option. In such
+ *!   cases it's possible that @[set_charset] works better (provided
+ *!   the server is 4.1 or newer).
  */
 static void f_create(INT32 args)
 {
@@ -617,11 +655,31 @@ static void f_create(INT32 args)
     }
   }
 
-  pop_n_elems(args);
-
   pike_mysql_set_ssl(PIKE_MYSQL->options);
 
-  pike_mysql_reconnect();
+  pike_mysql_reconnect (0);
+
+#ifndef HAVE_MYSQL_SET_CHARACTER_SET
+#ifdef HAVE_MYSQL_CHARACTER_SET_NAME
+  {
+    const char *charset = mysql_character_set_name (PIKE_MYSQL->socket);
+    if (PIKE_MYSQL->conn_charset)
+      free_string (PIKE_MYSQL->conn_charset);
+    if (charset)
+      PIKE_MYSQL->conn_charset = make_shared_string (charset);
+    else
+      /* Just paranoia; mysql_character_set_name should always return
+       * a string. */
+      PIKE_MYSQL->conn_charset = NULL;
+  }
+#else
+  if (PIKE_MYSQL->conn_charset)
+    free_string (PIKE_MYSQL->conn_charset);
+  PIKE_MYSQL->conn_charset = NULL;
+#endif
+#endif
+
+  pop_n_elems(args);
 }
 
 /*! @decl string _sprintf(int type, void|mapping flags)
@@ -642,34 +700,29 @@ static void mysql__sprintf(INT32 args)
   {
     case 'O':
     {
-      struct pike_string *res;
-      MYSQL *socket;
-      const char *info;
+      MYSQL *socket = PIKE_MYSQL->socket;
 
-      if(!PIKE_MYSQL->socket)
-	pike_mysql_reconnect();
-      socket = PIKE_MYSQL->socket;
-
-      MYSQL_ALLOW();
-      info = mysql_get_host_info(socket);
-      MYSQL_DISALLOW();
-
-      push_text("mysql(/* %s %s*/)");
-      push_text(info);
+      if (socket) {
+	const char *info;
+	MYSQL_ALLOW();
+	info = mysql_get_host_info(socket);
+	MYSQL_DISALLOW();
+	push_text("mysql(/*%s%s*/)");
+	push_text(info);
 #ifdef HAVE_MYSQL_SSL
-      if (PIKE_MYSQL->mysql->options.use_ssl) {
-	push_text("using SSL");
-      }
-      else push_text("");
+	if (PIKE_MYSQL->mysql->options.use_ssl) {
+	  push_text(", SSL");
+	}
+	else
+	  push_text("");
 #else
-      push_text("");
+	push_text("");
 #endif /* HAVE_MYSQL_SSL */
+	f_sprintf(3);
+      }
+      else
+	push_constant_text ("mysql()");
 
-      f_sprintf(3);
-
-      res = Pike_sp[-1].u.string;
-      Pike_sp--;
-      push_string(res);
       return;
     }
 
@@ -694,7 +747,7 @@ static void f_affected_rows(INT32 args)
   INT64 count;
 
   if (!PIKE_MYSQL->socket) {
-    pike_mysql_reconnect();
+    pike_mysql_reconnect (1);
   }
   pop_n_elems(args);
   socket = PIKE_MYSQL->socket;
@@ -717,7 +770,7 @@ static void f_insert_id(INT32 args)
   INT64 id;
 
   if (!PIKE_MYSQL->socket) {
-    pike_mysql_reconnect();
+    pike_mysql_reconnect (1);
   }
   pop_n_elems(args);
 
@@ -742,7 +795,7 @@ static void f_error(INT32 args)
   const char *error_msg;
 
   if (!PIKE_MYSQL->socket) {
-    pike_mysql_reconnect();
+    pike_mysql_reconnect (1);
   }
 
   socket = PIKE_MYSQL->socket;
@@ -794,7 +847,7 @@ static void f_select_db(INT32 args)
   }
   if (!socket || tmp) {
     /* The connection might have been closed. */
-    pike_mysql_reconnect();
+    pike_mysql_reconnect (1);
 
     socket = PIKE_MYSQL->socket;
 
@@ -823,19 +876,8 @@ static void f_select_db(INT32 args)
   pop_n_elems(args);
 }
 
-/*! @decl object(Mysql.mysql_result) big_query(string query)
- *!
- *! Make an SQL query.
- *!
- *! This function sends the SQL query @[query] to the Mysql-server. The result
- *! of the query is returned as a @[Mysql.mysql_result] object.
- *!
- *! Returns @expr{0@} (zero) if the query didn't return any result
- *! (e.g. @tt{INSERT@} or similar).
- *!
- *! @seealso
- *!   @[Mysql.mysql_result]
- */
+#define PIKE_MYSQL_FLAG_STORE_RESULT	1
+
 static void f_big_query(INT32 args)
 {
   MYSQL *socket = PIKE_MYSQL->socket;
@@ -845,12 +887,12 @@ static void f_big_query(INT32 args)
   int tmp = -1;
 
   if (!args) {
-    SIMPLE_TOO_FEW_ARGS_ERROR ("Mysql.mysql->big_query", 1);
+    SIMPLE_TOO_FEW_ARGS_ERROR ("big_query", 1);
   }
 #ifdef HAVE_MYSQL_REAL_QUERY
-  CHECK_8BIT_STRING ("Mysql.mysql->big_query", 1);
+  CHECK_8BIT_STRING ("big_query", 1);
 #else
-  CHECK_8BIT_NONBINARY_STRING ("Mysql.mysql->big_query", 1);
+  CHECK_8BIT_NONBINARY_STRING ("big_query", 1);
 #endif
 
   query = sp[-args].u.string->str;
@@ -885,7 +927,7 @@ static void f_big_query(INT32 args)
   }
   if (!socket) {
     /* The connection might have been closed. */
-    pike_mysql_reconnect();
+    pike_mysql_reconnect (1);
 
     socket = PIKE_MYSQL->socket;
 
@@ -912,10 +954,10 @@ static void f_big_query(INT32 args)
     MYSQL_DISALLOW();
 
     if (sp[-args].u.string->len <= 512) {
-      Pike_error("Mysql.mysql->big_query(): Query \"%s\" failed (%s)\n",
-	    sp[-args].u.string->str, err);
+      Pike_error("big_query(): Query \"%s\" failed (%s)\n",
+		 query, err);
     } else {
-      Pike_error("Mysql.mysql->big_query(): Query failed (%s)\n", err);
+      Pike_error("big_query(): Query failed (%s)\n", err);
     }
   }
 
@@ -939,8 +981,12 @@ static void f_big_query(INT32 args)
     MYSQL_DISALLOW();
 
     if (err) {
-      Pike_error("Mysql.mysql->big_query(): Couldn't create result for query (%s)\n",
-		 mysql_error(socket));
+      const char *msg;
+      MYSQL_ALLOW();
+      msg = mysql_error (socket);
+      MYSQL_DISALLOW();
+      Pike_error("big_query(): Couldn't create result for query (%s)\n",
+		 msg);
     }
     /* query was INSERT or similar - return 0 */
 
@@ -958,11 +1004,13 @@ static void f_big_query(INT32 args)
     if ((!(res = (struct precompiled_mysql_result *)
 	   get_storage(o, mysql_result_program))) || res->result) {
       mysql_free_result(result);
-      Pike_error("Mysql.mysql->big_query(): Bad mysql result object!\n");
+      Pike_error("big_query(): Bad mysql result object!\n");
     }
     res->result = result;
   }
 }
+
+
 /*! @decl void create_db(string database)
  *!
  *! Create a new database
@@ -1003,7 +1051,7 @@ static void f_create_db(INT32 args)
   }
   if (!socket || tmp) {
     /* The connection might have been closed */
-    pike_mysql_reconnect();
+    pike_mysql_reconnect (1);
 
     socket = PIKE_MYSQL->socket;
 
@@ -1061,7 +1109,7 @@ static void f_drop_db(INT32 args)
   }
   if (!socket || tmp) {
     /* The connection might have been closed */
-    pike_mysql_reconnect();
+    pike_mysql_reconnect (1);
 
     socket = PIKE_MYSQL->socket;
 
@@ -1109,7 +1157,7 @@ static void f_shutdown(INT32 args)
   }
   if (!socket || tmp) {
     /* The connection might have been closed */
-    pike_mysql_reconnect();
+    pike_mysql_reconnect (1);
 
     socket = PIKE_MYSQL->socket;
 
@@ -1155,7 +1203,7 @@ static void f_reload(INT32 args)
   }
   if (!socket || tmp) {
     /* The connection might have been closed */
-    pike_mysql_reconnect();
+    pike_mysql_reconnect (1);
 
     socket = PIKE_MYSQL->socket;
 
@@ -1188,7 +1236,7 @@ static void f_statistics(INT32 args)
   const char *stats;
 
   if (!socket) {
-    pike_mysql_reconnect();
+    pike_mysql_reconnect (1);
     socket = PIKE_MYSQL->socket;
   }
 
@@ -1216,7 +1264,7 @@ static void f_server_info(INT32 args)
   const char *info;
 
   if (!socket) {
-    pike_mysql_reconnect();
+    pike_mysql_reconnect (1);
     socket = PIKE_MYSQL->socket;
   }
 
@@ -1247,7 +1295,7 @@ static void f_host_info(INT32 args)
   const char *info;
 
   if (!PIKE_MYSQL->socket) {
-    pike_mysql_reconnect();
+    pike_mysql_reconnect (1);
   }
 
   socket = PIKE_MYSQL->socket;
@@ -1279,7 +1327,7 @@ static void f_protocol_info(INT32 args)
   int prot;
 
   if (!PIKE_MYSQL->socket) {
-    pike_mysql_reconnect();
+    pike_mysql_reconnect (1);
   }
 
   pop_n_elems(args);
@@ -1335,7 +1383,7 @@ static void f_list_dbs(INT32 args)
   }
   if (!socket || !result) {
     /* The connection might have been closed */
-    pike_mysql_reconnect();
+    pike_mysql_reconnect (1);
 
     socket = PIKE_MYSQL->socket;
 
@@ -1421,7 +1469,7 @@ static void f_list_tables(INT32 args)
   }
   if (!socket || !result) {
     /* The connection might have been closed */
-    pike_mysql_reconnect();
+    pike_mysql_reconnect (1);
 
     socket = PIKE_MYSQL->socket;
 
@@ -1572,7 +1620,7 @@ static void f_list_fields(INT32 args)
   }
   if (!socket || !result) {
     /* The connection might have been closed */
-    pike_mysql_reconnect();
+    pike_mysql_reconnect (1);
 
     socket = PIKE_MYSQL->socket;
 
@@ -1632,7 +1680,7 @@ static void f_list_processes(INT32 args)
   }
   if (!socket || !result) {
     /* The connection might have been closed */
-    pike_mysql_reconnect();
+    pike_mysql_reconnect (1);
 
     socket = PIKE_MYSQL->socket;
 
@@ -1695,6 +1743,107 @@ static void f_binary_data(INT32 args)
 #endif /* HAVE_MYSQL_FETCH_LENGTHS */
 }
 
+static void f_set_charset (INT32 args)
+{
+  struct pike_string *charset;
+  get_all_args ("set_charset", args, "%n", &charset);
+  if (string_has_null (charset))
+    SIMPLE_ARG_ERROR ("set_charset", 0,
+		      "The charset name cannot contain a NUL character.");
+
+#ifdef HAVE_MYSQL_SET_CHARACTER_SET
+  {
+    int res;
+    MYSQL *socket = PIKE_MYSQL->socket;
+    MYSQL_ALLOW();
+    res = mysql_set_character_set (socket, charset->str);
+    MYSQL_DISALLOW();
+    if (res) {
+      const char *err;
+      MYSQL_ALLOW();
+      err = mysql_error(socket);
+      MYSQL_DISALLOW();
+      Pike_error("Setting the charset failed: %s\n", err);
+    }
+  }
+
+#else  /* !HAVE_MYSQL_SET_CHARACTER_SET */
+  push_constant_text ("SET NAMES '");
+  ref_push_string (charset);
+  push_constant_text ("'");
+  f_add (3);
+  f_big_query(1);
+  args++;
+  if (PIKE_MYSQL->conn_charset)
+    free_string (PIKE_MYSQL->conn_charset);
+  copy_shared_string (PIKE_MYSQL->conn_charset, charset);
+#endif
+
+  pop_n_elems (args);
+}
+
+static void f_get_charset (INT32 args)
+{
+  pop_n_elems (args);
+#ifdef HAVE_MYSQL_SET_CHARACTER_SET
+  {
+    /* mysql_character_set_name should always exist if
+     * mysql_set_character_set exists. */
+    const char *charset = mysql_character_set_name (PIKE_MYSQL->socket);
+    if (charset)
+      push_text (charset);
+    else
+      /* Just paranoia; mysql_character_set_name should always return
+       * a string. */
+      push_constant_text ("latin1");
+  }
+#else
+  if (PIKE_MYSQL->conn_charset)
+    ref_push_string (PIKE_MYSQL->conn_charset);
+  else
+    push_constant_text ("latin1");
+#endif
+}
+
+static void f__can_send_as_latin1 (INT32 args)
+/* Helper function to detect if a string can be sent in the latin1
+ * encoding. */
+{
+  struct pike_string *str;
+  ptrdiff_t i;
+  int res;
+
+  if (args != 1)
+    SIMPLE_WRONG_NUM_ARGS_ERROR ("_can_send_as_latin1", 1);
+  if (Pike_sp[-1].type != T_STRING)
+    SIMPLE_ARG_TYPE_ERROR ("_can_send_as_latin1", 0, "string");
+  str = Pike_sp[-1].u.string;
+
+  if (str->size_shift)
+    res = 0;
+
+  else {
+    /* Have to go through the string to check that it doesn't contain
+     * any of those pesky chars in the 0x80..0x9f range that MySQL has
+     * remapped in latin1. */
+    /* This check could be made tighter by ignoring chars in strings
+     * with introducers. */
+    res = 1;
+    for (i = str->len; i--;) {
+      int chr = STR0 (str)[i];
+      if (chr >= 0x80 && chr <= 0x9f &&
+	  chr != 0x81 && chr != 0x8d && chr != 0x8f &&
+	  chr != 0x90 && chr != 0x9d) {
+	res = 0;
+	break;
+      }
+    }
+  }
+
+  pop_stack();
+  push_int (res);
+}
+
 /*! @endclass
  */
 
@@ -1727,7 +1876,7 @@ PIKE_MODULE_INIT
    * program->refs++;
    *
    */
- 
+
   start_new_program();
   ADD_STORAGE(struct precompiled_mysql);
 
@@ -1778,10 +1927,17 @@ PIKE_MODULE_INIT
   /* function(void:int) */
   ADD_FUNCTION("binary_data", f_binary_data,tFunc(tVoid,tInt), ID_PUBLIC);
 
+  ADD_FUNCTION ("set_charset", f_set_charset, tFunc(tStr,tVoid), ID_PUBLIC);
+  ADD_FUNCTION ("get_charset", f_get_charset, tFunc(tVoid,tStr), ID_PUBLIC);
+  ADD_FUNCTION ("_can_send_as_latin1", f__can_send_as_latin1,
+		tFunc(tStr,tInt01), ID_STATIC);
+
   add_integer_constant( "CLIENT_COMPRESS", CLIENT_COMPRESS, 0);
   add_integer_constant( "CLIENT_FOUND_ROWS", CLIENT_FOUND_ROWS, 0);
   add_integer_constant( "CLIENT_IGNORE_SPACE", CLIENT_IGNORE_SPACE, 0);
+#ifdef HAVE_CLIENT_INTERACTIVE
   add_integer_constant( "CLIENT_INTERACTIVE", CLIENT_INTERACTIVE, 0);
+#endif
   add_integer_constant( "CLIENT_NO_SCHEMA", CLIENT_NO_SCHEMA, 0);
   add_integer_constant( "CLIENT_ODBC", CLIENT_ODBC, 0);
 
