@@ -1,6 +1,11 @@
 /*
- * $Id: acconfig.h,v 1.6 2003/12/23 17:31:43 grubba Exp $
- *
+|| This file is part of Pike. For copyright information see COPYRIGHT.
+|| Pike is distributed under GPL, LGPL and MPL. See the file COPYING
+|| for more information.
+|| $Id: acconfig.h,v 1.9 2003/12/23 17:30:17 grubba Exp $
+*/
+
+/*
  * Config-file for the Pike ODBC-module.
  *
  * Henrik Grubbstrm
@@ -29,6 +34,15 @@
 /* Define if you have SQLULEN */
 #undef HAVE_SQLULEN
 
+/* Define if you have SQLHENV */
+#undef HAVE_SQLHENV
+
+/* Define if you have SQLHDBC */
+#undef HAVE_SQLHDBC
+
+/* Define if you have SQLHSTMT */
+#undef HAVE_SQLHSTMT
+
 /* Define if you have ODBC */
 #undef HAVE_ODBC
 
