@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: precompiled_odbc.h,v 1.18 2002/10/11 01:39:47 nilsson Exp $
+|| $Id: precompiled_odbc.h,v 1.22 2006/03/25 11:34:30 grubba Exp $
 */
 
 /*
@@ -88,6 +88,18 @@ typedef SQLINTEGER	SQLLEN;
 typedef SQLUINTEGER	SQLULEN;
 #endif /* !HAVE_SQLLEN */
 
+#ifndef HAVE_SQLHENV
+typedef HENV	SQLHENV;
+#endif /* !HAVE_SQLENV */
+
+#ifndef HAVE_SQLHDBC
+typedef HDBC	SQLHDBC;
+#endif /* !HAVE_SQLHDBC */
+
+#ifndef HAVE_SQLHSTMT
+typedef HSTMT	SQLHSTMT;
+#endif /* !HAVE_SQLSTMT */
+
 /*
  * Structures
  */
@@ -97,7 +109,7 @@ struct field_info {
 };
 
 struct precompiled_odbc {
-  HDBC hdbc;
+  SQLHDBC hdbc;
   SQLLEN affected_rows;
   unsigned int flags;
   struct pike_string *last_error;
@@ -106,7 +118,7 @@ struct precompiled_odbc {
 struct precompiled_odbc_result {
   struct object *obj;
   struct precompiled_odbc *odbc;
-  HSTMT hstmt;
+  SQLHSTMT hstmt;
   SWORD num_fields;
   SQLLEN num_rows;
   struct array *fields;
@@ -126,9 +138,12 @@ struct precompiled_odbc_result {
 /*
  * Prototypes
  */
+#ifdef SQL_WCHAR
+void push_sqlwchar(SQLWCHAR *str, size_t num_bytes);
+#endif /* SQL_WCHAR */
 void odbc_error(const char *fun, const char *msg,
-		struct precompiled_odbc *odbc, HSTMT hstmt,
-		RETCODE code, void (*clean)(void));
+		struct precompiled_odbc *odbc, SQLHSTMT hstmt,
+		RETCODE code, void (*clean)(void *), void *clean_arg);
 
 void init_odbc_res_programs(void);
 void exit_odbc_res(void);
