@@ -1,6 +1,14 @@
+/*
+|| This file is part of Pike. For copyright information see COPYRIGHT.
+|| Pike is distributed under GPL, LGPL and MPL. See the file COPYING
+|| for more information.
+|| $Id: pipe.c,v 1.58 2003/12/12 17:45:51 nilsson Exp $
+*/
+
 #include "global.h"
 #include "config.h"
 #include "machine.h"
+#include "module.h"
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -30,7 +38,7 @@
 
 #include <fcntl.h>
 
-RCSID("$Id: pipe.c,v 1.46 2003/10/07 16:27:19 mast Exp $");
+RCSID("$Id: pipe.c,v 1.58 2003/12/12 17:45:51 nilsson Exp $");
 
 #include "threads.h"
 #include "stralloc.h"
@@ -51,9 +59,8 @@ RCSID("$Id: pipe.c,v 1.46 2003/10/07 16:27:19 mast Exp $");
 #endif
 #endif
 
-/* must be included last */
-#include "module_magic.h"
 
+#define sp Pike_sp
 
 /*
 #define PIPE_STRING_DEBUG
@@ -377,7 +384,7 @@ static int read_some_data(void)
   struct input * i = this->firstinput;
 
   if (!i || i->type != I_BLOCKING_OBJ) {
-    fatal("PIPE: read_some_data(): Bad input type!\n");
+    Pike_fatal("PIPE: read_some_data(): Bad input type!\n");
     return -1;
   }
   push_int(8192);
@@ -472,6 +479,7 @@ static INLINE struct pike_string* gimme_some_data(size_t pos)
 	  if (errno != EINTR) {
 	    return(NULL);
 	  }
+	  check_threads_etc();
 	}
       } while(len < 0);
       /*
@@ -689,8 +697,6 @@ static void pipe_input(INT32 args)
 {
    struct input *i;
    int fd=-1;			/* Per, one less warning to worry about... */
-   char *m;
-   struct stat s;
    struct object *obj;
 
    if (args<1 || sp[-args].type != T_OBJECT)
@@ -711,6 +717,9 @@ static void pipe_input(INT32 args)
    /* We do not handle mmaps if we have a buffer */
    if(THIS->fd == -1)
    {
+     char *m;
+     struct stat s;
+
      apply(obj, "query_fd", 0);
      if(sp[-1].type == T_INT) fd=sp[-1].u.integer;
      pop_stack();
@@ -720,12 +729,9 @@ static void pipe_input(INT32 args)
        off_t filep=fd_lseek(fd, 0L, SEEK_CUR); /* keep the file pointer */
        size_t len = s.st_size - filep;
        if(S_ISREG(s.st_mode)	/* regular file */
-	  && ((long)(m=(char *)mmap(0, len, PROT_READ,
-				    MAP_FILE|MAP_SHARED,fd,filep))!=-1))
+	  && ((m=(char *)mmap(0, len, PROT_READ,
+			      MAP_FILE|MAP_SHARED,fd,filep))+1))
        {
-#ifdef HAVE_GETEUID
-	 int ou = 0;
-#endif
 	 mmapped += len;
 
 	 i->type=I_MMAP;
@@ -733,35 +739,7 @@ static void pipe_input(INT32 args)
 	 i->u.mmap=m;
 #if defined(HAVE_MADVISE) && defined(MADV_SEQUENTIAL)
 	 /* Mark the pages as sequential read only access... */
-
-	 /* NOTE:
-	  *
-	  *	Potential race-condition with other threads
-	  */
-
-#ifdef HAVE_GETEUID
-	 if((ou=geteuid()) && !getuid()) {
-#ifdef HAVE_SETEUID
-	   seteuid(0);
-#else /* ! HAVE_SETEUID */
-#ifdef HAVE_SETRESUID
-	   setresuid(-1, 0, -1);
-#endif /* HAVE_SETRESUID */
-#endif /* HAVE_SETEUID */
-	 }
-#endif
 	 madvise(m, len, MADV_SEQUENTIAL);
-#ifdef HAVE_GETEUID
-	 if(ou) {
-#ifdef HAVE_SETEUID
-	   seteuid(ou);
-#else /* ! HAVE_SETEUID */
-#ifdef HAVE_SETRESUID
-	   setresuid(-1, ou, -1);
-#endif /* HAVE_SETRESUID */
-#endif /* HAVE_SETEUID */
-	 }
-#endif
 #endif
 	 pop_n_elems(args);
 	 push_int(0);
@@ -881,14 +859,6 @@ static void pipe_output(INT32 args)
     {
       /* keep the file pointer of the duped fd */
       THIS->pos=fd_lseek(fd, 0L, SEEK_CUR);
-      
-#if 0
-      /* This won't work if the spider-module is dynamically linked. */
-      push_int(THIS->fd);
-      push_string(make_shared_string("pipe.c: file buffer"));
-      f_mark_fd(2);
-      pop_stack();
-#endif /* 0 */
 
       THIS->living_outputs++;
 
@@ -1132,7 +1102,7 @@ static void pipe_close_input_callback(INT32 args)
 static void pipe_version(INT32 args)
 {
    pop_n_elems(args);
-   push_string(make_shared_string("PIPE ver 2.0"));
+   push_text("PIPE ver 2.0");
 }
 
 /********** init/exit *******************************************************/
@@ -1289,13 +1259,11 @@ void f__pipe_debug(INT32 args)
   f_aggregate(7);
 }
 
-void pike_module_init(void)
+PIKE_MODULE_INIT
 {
    start_new_program();
    ADD_STORAGE(struct pipe);
    
-/* function(:array) */
-  ADD_EFUN("_pipe_debug", f__pipe_debug,tFunc(tNone,tArray), 0);
    /* function(object:void) */
   ADD_FUNCTION("input",pipe_input,tFunc(tObj,tVoid),0);
    /* function(object,void|int:void) */
@@ -1350,19 +1318,15 @@ void pike_module_init(void)
    set_exit_callback(exit_output_struct);
    output_program=end_program();
    add_program_constant("__output",output_program, 0);
+
+   /* function(:array) */
+  ADD_FUNCTION("_pipe_debug", f__pipe_debug,tFunc(tNone,tArray), 0);
 }
 
-void pike_module_exit(void) 
+PIKE_MODULE_EXIT
 {
   if(pipe_program) free_program(pipe_program);
   pipe_program=0;
   if(output_program) free_program(output_program);
   output_program=0;
 }
-
-
-
-
-
-
-
