@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: image_gif.c,v 1.14 2002/10/21 17:06:25 marcus Exp $
+|| $Id: image_gif.c,v 1.20 2004/01/22 23:13:35 nilsson Exp $
 */
 
 /*
@@ -39,7 +39,7 @@
 #include <ctype.h>
 
 #include "stralloc.h"
-RCSID("$Id: image_gif.c,v 1.14 2002/10/21 17:06:25 marcus Exp $");
+RCSID("$Id: image_gif.c,v 1.20 2004/01/22 23:13:35 nilsson Exp $");
 #include "pike_macros.h"
 #include "object.h"
 #include "constants.h"
@@ -322,7 +322,7 @@ void image_gif_header_block(INT32 args)
 void image_gif_end_block(INT32 args)
 {
    pop_n_elems(args);
-   push_string(make_shared_string("\x3b"));
+   push_constant_text("\x3b");
 }
 
 /*
@@ -379,12 +379,12 @@ static void image_gif__gce_block(INT32 args)
 	   0x21, /* extension intruder */
 	   0xf9, /* gce extension */
 	   4, /* block size */
-	   (((sp[4-args].u.integer & 7)<<2) /* disposal */
-	    | ((!!sp[3-args].u.integer)<<1) /* user input */
-	    | (!!sp[-args].u.integer)), /* transparency */
-	   sp[2-args].u.integer & 255, /* delay, ls8 */
-	   (sp[2-args].u.integer>>8) & 255, /* delay, ms8 */
-	   sp[1-args].u.integer & 255, /* transparency index */
+	   ((((int) sp[4-args].u.integer & 7)<<2) /* disposal */
+	    | ((!!(int) sp[3-args].u.integer)<<1) /* user input */
+	    | (!!(int) sp[-args].u.integer)), /* transparency */
+	   (int) sp[2-args].u.integer & 255, /* delay, ls8 */
+	   ((int) sp[2-args].u.integer>>8) & 255, /* delay, ms8 */
+	   (int) sp[1-args].u.integer & 255, /* transparency index */
 	   0 /* end block */
 	   );
 	   
@@ -2219,7 +2219,7 @@ void image_gif_decode_layer(INT32 args)
 }
 
 /*
-**! method mapping decode_map(INT32 args)
+**! method mapping decode_map(string|array layers)
 **!	Returns a mapping similar to other decoders
 **!	<tt>_decode</tt> function.
 **!
@@ -2235,7 +2235,7 @@ void image_gif_decode_layer(INT32 args)
 **!     </pre>
 **!
 **! note:
-**!	The wierd name of this function (not <tt>_decode</tt>
+**!	The weird name of this function (not <tt>_decode</tt>
 **!	as the other decoders) is because gif was the first
 **!	decoder and was written before the API was finally
 **!	defined. Sorry about that. /Mirar
@@ -2368,7 +2368,7 @@ void image_gif__encode_extension(INT32 args)
        a->item[2].type!=T_STRING)
       Pike_error("Image.GIF._encode_extension: Illegal type in indices 1 or 2\n");
 
-   sprintf(buf,"%c%c",0x21,a->item[1].u.integer);
+   sprintf(buf,"%c%c",0x21,(int) a->item[1].u.integer);
    push_string(make_shared_binary_string(buf,2));
 
    n=1;
@@ -2417,7 +2417,7 @@ void image_gif__encode(INT32 args)
 
    if (args<1 ||
        sp[-args].type!=T_ARRAY)
-      Pike_error("Image.GIF._encode: Illegal argument (expected array)");
+      Pike_error("Image.GIF._encode: Illegal argument (expected array)\n");
 
    add_ref(a=sp[-args].u.array);
    pos=0;
@@ -2739,22 +2739,21 @@ PIKE_MODULE_INIT
 {
 #ifdef DYNAMIC_MODULE
   /* These could be re-written to use PIKE_MODULE_IMPORT */
-   push_string(make_shared_string("Image"));
-   push_int(0);
-   SAFE_APPLY_MASTER("resolv",2);
+   push_text("Image");
+   SAFE_APPLY_MASTER("resolv",1);
    if (sp[-1].type==T_OBJECT) 
    {
       stack_dup();
       stack_dup();
-      push_string(make_shared_string("Image"));
+      push_text("Image");
       f_index(2);
       image_program=program_from_svalue(sp-1);
       pop_stack();
-      push_string(make_shared_string("Colortable"));
+      push_text("Colortable");
       f_index(2);
       image_colortable_program=program_from_svalue(sp-1);
       pop_stack();
-      push_string(make_shared_string("Layer"));
+      push_text("Layer");
       f_index(2);
       image_layer_program=program_from_svalue(sp-1);
    }
