@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: transupp.c,v 1.11 2002/10/17 09:19:16 nilsson Exp $
+|| $Id: transupp.c,v 1.13 2003/04/30 18:03:14 grubba Exp $
 */
 
 #include "global.h"
@@ -55,6 +55,18 @@
 #undef FAR
 #endif
 
+#ifdef HAVE_JCONFIG_H_HAVE_BOOLEAN
+#ifdef HAVE_WINDOWS_H
+#include <windows.h>
+#ifdef HAVE_WTYPES_H
+/* jconfig.h has probably been compiled without WIN32_LEAN_AND_MEAN...
+ * So we need this one to get the boolean typedef.
+ */
+#include <wtypes.h>
+#endif
+#endif
+#endif
+
 #include <jerror.h>
 #include "jinclude.h"
 #include <jpeglib.h>
