@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: nt.c,v 1.57 2003/06/26 12:41:46 grubba Exp $
+|| $Id: nt.c,v 1.63 2004/04/06 13:00:50 nilsson Exp $
 */
 
 /*
@@ -18,7 +18,11 @@
 
 #ifdef __NT__
 
+#ifdef HAVE_WINSOCK2_H
+#include <winsock2.h>
+#else
 #include <winsock.h>
+#endif
 #include <windows.h>
 #include <accctrl.h>
 #include <lm.h>
@@ -52,6 +56,7 @@
 #include "interpret.h"
 #include "operators.h"
 #include "stuff.h"
+#include "pike_security.h"
 
 #define sp Pike_sp
 
@@ -171,11 +176,11 @@ static void throw_nt_error(char *funcname, int err)
   Pike_error("%s: %s\n", funcname, msg);
 }
 
-
 static void f_cp(INT32 args)
 {
   char *from, *to;
   int ret;
+  VALID_FILE_IO("cp","write");
   get_all_args("cp",args,"%s%s",&from,&to);
   ret=CopyFile(from, to, 0);
   if(!ret) errno=GetLastError();
@@ -211,7 +216,7 @@ static void push_regvalue(DWORD type, char* buffer, DWORD len)
 				 DO_NOT_WARN((DWORD)(sizeof(buffer)-len-1)));
       if(type>sizeof(buffer)-len-1 || !type)
 	Pike_error("RegGetValue: Failed to expand data.\n");
-      push_string(make_shared_string(buffer+len));
+      push_text(buffer+len);
       break;
       
     case REG_MULTI_SZ:
@@ -252,6 +257,7 @@ static const HKEY hkeys[] = {
   HKEY_LOCAL_MACHINE,
   HKEY_CURRENT_USER,
   HKEY_USERS,
+  /* HKEY_CURRENT_CONFIG */
 };
 
 /*! @decl string|int|array(string) RegGetValue(int hkey, string key, @
@@ -293,7 +299,7 @@ void f_RegGetValue(INT32 args)
   DWORD len,type;
   char buffer[8192];
   len=sizeof(buffer)-1;
-  get_all_args("RegQueryValue", args, "%d%s%s",
+  get_all_args("RegQueryValue", args, "%i%s%s",
 	       &hkey_num, &key, &ind);
 
   if ((hkey_num < 0) || (hkey_num >= NELEM(hkeys))) {
@@ -335,12 +341,21 @@ static void do_regclosekey(HKEY key)
  *!   @endint
  *!
  *! @param key
- *!   Registry key.
+ *!   A registry key.
  *!
  *! @returns
- *!   Returns an array of value keys stored at the specified location iof any.
+ *!   Returns an array of value keys stored at the specified location if any.
  *!   Throws errors on failure.
  *!
+ *! @example
+ *!   > RegGetKeyNames(HKEY_CURRENT_USER, "Keyboard Layout");
+ *!  (1) Result: ({
+ *!    "IMEtoggle",
+ *!    "Preload",
+ *!    "Substitutes",
+ *!    "Toggle"
+ *!   })
+ *!
  *! @note
  *!   This function is only available on Win32 systems.
  *!
@@ -354,7 +369,7 @@ void f_RegGetKeyNames(INT32 args)
   int i,ret;
   HKEY new_key;
   ONERROR tmp;
-  get_all_args("RegGetKeyNames", args, "%d%s",
+  get_all_args("RegGetKeyNames", args, "%i%s",
 	       &hkey_num, &key);
 
   if ((hkey_num < 0) || (hkey_num >= NELEM(hkeys))) {
@@ -417,6 +432,12 @@ void f_RegGetKeyNames(INT32 args)
  *!   Returns a mapping with all the values stored at the specified location
  *!   in the register if any. Throws errors on failure.
  *!
+ *! @example
+ *! > RegGetValues(HKEY_CURRENT_USER, "Keyboard Layout\\Preload");
+ *!(5) Result: ([
+ *!  "1":"0000041d"
+ *! ])
+ *!
  *! @note
  *!   This function is only available on Win32 systems.
  *!
@@ -431,7 +452,7 @@ void f_RegGetValues(INT32 args)
   HKEY new_key;
   ONERROR tmp;
 
-  get_all_args("RegGetValues", args, "%d%s",
+  get_all_args("RegGetValues", args, "%i%s",
 	       &hkey_num, &key);
 
   if ((hkey_num < 0) || (hkey_num >= NELEM(hkeys))) {
@@ -640,7 +661,9 @@ void f_LogonUser(INT32 args)
   DWORD logontype, logonprovider;
   HANDLE x;
   BOOL ret;
-    
+
+  ASSERT_SECURITY_ROOT("System.LogonUser");
+
   check_all_args("System.LogonUser",args,
 		 BIT_STRING, BIT_INT | BIT_STRING, BIT_STRING,
 		 BIT_INT | BIT_VOID, BIT_INT | BIT_VOID,0);
@@ -2544,6 +2567,7 @@ static void f_GetFileAttributes(INT32 args)
 {
   char *file;
   DWORD ret;
+  VALID_FILE_IO("GetFileAttributes","read");
   get_all_args("GetFileAttributes",args,"%s",&file);
   ret=GetFileAttributes( (LPCTSTR) file);
   pop_stack();
@@ -2566,7 +2590,8 @@ static void f_SetFileAttributes(INT32 args)
   char *file;
   INT_TYPE attr, ret;
   DWORD tmp;
-  get_all_args("SetFileAttributes", args, "%s%d", &file, &attr);
+  VALID_FILE_IO("SetFileAttributes","write");
+  get_all_args("SetFileAttributes", args, "%s%i", &file, &attr);
   tmp=attr;
   ret=SetFileAttributes( (LPCTSTR) file, tmp);
   pop_stack();
@@ -2801,6 +2826,7 @@ static void f_SetNamedSecurityInfo(INT32 args)
   DWORD ret;
   SE_OBJECT_TYPE type=SE_FILE_OBJECT;
 
+  ASSERT_SECURITY_ROOT("SetNamedSecurity");
   get_all_args("SetNamedSecurityInfo",args,"%s%m",&name,&m);
 
   if((sval=simple_mapping_string_lookup(m, "type")))
@@ -3407,7 +3433,7 @@ static void f_sctx_getusername(INT32 args)
     return;
   }
 
-  push_string(make_shared_string(name.sUserName));
+  push_text(name.sUserName);
 
   freecontextbuffer(name.sUserName);
 }
@@ -3433,8 +3459,8 @@ static void f_sctx_getlasterror(INT32 args)
                 NULL 
                 );
   sprintf(buf, "0x%04x: ", sctx->lastError);
-  push_string(make_shared_string(buf));
-  push_string(make_shared_string(lpMsgBuf));
+  push_text(buf);
+  push_text(lpMsgBuf);
   f_add(2);
   LocalFree(lpMsgBuf);
 }
