@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: program.h,v 1.181 2004/03/23 13:41:56 grubba Exp $
+|| $Id: program.h,v 1.205 2004/09/28 23:58:35 mast Exp $
 */
 
 #ifndef PROGRAM_H
@@ -10,31 +10,27 @@
 
 #include <stdarg.h>
 #include "global.h"
-#include "pike_types.h"
 #include "pike_macros.h"
+#include "pike_error.h"
 #include "svalue.h"
+#include "dmalloc.h"
 #include "time_stuff.h"
 #include "program_id.h"
 #include "block_alloc_h.h"
 
-#define STRUCT
-#include "compilation.h"
-
-#define EXTERN
-#include "compilation.h"
-
 /* Needed to support dynamic loading on NT */
 PMOD_PROTO extern struct program_state * Pike_compiler;
 
 /* Compilation flags */
-#define COMPILATION_CHECK_FINAL		0x01
-	/* This flag is set when resolve functions should force the lookup so
-	 * that we don't get a placeholder back. Used for inherits. */
-#define COMPILATION_FORCE_RESOLVE	0x02
+#define COMPILATION_CHECK_FINAL         0x01
+        /* This flag is set when resolve functions should force the lookup so
+         * that we don't get a placeholder back. Used for inherits. */
+#define COMPILATION_FORCE_RESOLVE       0x02
+
 /* #define FORCE_RESOLVE_DEBUG */
 /* Helper macros for force_resolve */
 #ifdef FORCE_RESOLVE_DEBUG
-#define DO_IF_FRD(X)	X
+#define DO_IF_FRD(X)  X
 #else /* !FORCE_RESOLVE_DEBUG */
 #define DO_IF_FRD(X)
 #endif /* FORCE_RESOLVE_DEBUG */
@@ -42,15 +38,15 @@ PMOD_PROTO extern struct program_state * Pike_compiler;
     int tmp_ = (OLD) = Pike_compiler->flags;				\
     Pike_compiler->flags |= COMPILATION_FORCE_RESOLVE;			\
     DO_IF_FRD(fprintf(stderr,						\
-		      "Force resolve on. Flags:0x%04x (0x%04x)\n",	\
-		      Pike_compiler->flags, tmp_));			\
+		      "Force resolve on. Flags:0x%04x (0x%04x)\n",      \
+		      Pike_compiler->flags, tmp_));                     \
   } while(0)
 #define UNSET_FORCE_RESOLVE(OLD) do {					\
     int tmp_ = (Pike_compiler->flags & ~COMPILATION_FORCE_RESOLVE) |	\
       ((OLD) & COMPILATION_FORCE_RESOLVE);				\
     DO_IF_FRD(fprintf(stderr,						\
 		      "Force resolve unset. Flags:0x%04x (0x%04x)\n",	\
-		      tmp_, Pike_compiler->flags));			\
+		      tmp_, Pike_compiler->flags));                     \
     Pike_compiler->flags = tmp_;					\
   } while(0)
 
@@ -103,18 +99,19 @@ extern struct pike_string *this_program_string;
 
 #define NUM_LFUNS 44
 
+/* NOTE: After this point there are only fake lfuns.
+ *       ie use low_find_lfun(), and NOT FIND_LFUN()!
+ */
+#define LFUN__SEARCH 44
+
 extern char *lfun_names[];
 
 extern struct pike_string *lfun_strings[];
 
-#ifndef STRUCT_SVALUE_DECLARED
-#define STRUCT_SVALUE_DECLARED
-struct svalue;
-#endif
-
 #ifndef STRUCT_NODE_S_DECLARED
 #define STRUCT_NODE_S_DECLARED
 struct node_s;
+typedef struct node_s node;
 #endif
 
 #ifndef STRUCT_OBJECT_DECLARED
@@ -122,6 +119,12 @@ struct node_s;
 struct object;
 #endif
 
+#define STRUCT
+#include "compilation.h"
+
+#define EXTERN
+#include "compilation.h"
+
 /* Byte-code method identification. */
 #define PIKE_BYTECODE_PORTABLE	-1	/* Only used by the codec. */
 #define PIKE_BYTECODE_DEFAULT	0
@@ -174,7 +177,23 @@ struct object;
 
 union idptr
 {
+  /* C function pointer. */
   void (*c_fun)(INT32);
+
+  /* For variables: Offset of the variable in the storage pointed to
+   * by inherit.storage_offset in the struct inherit that corresponds
+   * to the identifier. See LOW_GET_GLOBAL and GET_GLOBAL. The stored
+   * variable may be either a normal or a short svalue, depending on
+   * identifier.run_time_type.
+   *
+   * For constants: Offset of the struct program_constant in
+   * program.constants in the program pointed to by prog in the struct
+   * inherit that corresponds to the identifier.
+   *
+   * For pike functions: Offset to the start of the function in
+   * program.program in the program pointed to by prog in the struct
+   * inherit that corresponds to the identifier. Or -1 if a prototype.
+   */
   ptrdiff_t offset;
 };
 
@@ -186,7 +205,8 @@ union idptr
 #define IDENTIFIER_TYPE_MASK 7
 
 #define IDENTIFIER_VARARGS 8	/* Used for functions only. */
-#define IDENTIFIER_HAS_BODY 16	/* Function has a body (set already in pass 1). */
+#define IDENTIFIER_NO_THIS_REF 8 /* Used for variables only: Don't count refs to self. */
+#define IDENTIFIER_HAS_BODY 16  /* Function has a body (set already in pass 1). */
 #define IDENTIFIER_SCOPED 32   /* This is used for local functions only */
 #define IDENTIFIER_SCOPE_USED 64 /* contains scoped local functions */
 #define IDENTIFIER_ALIAS 128   /* This identifier is an alias. */
@@ -195,7 +215,7 @@ union idptr
 #define IDENTIFIER_IS_PIKE_FUNCTION(X) ((X) & IDENTIFIER_PIKE_FUNCTION)
 #define IDENTIFIER_IS_C_FUNCTION(X) ((X) & IDENTIFIER_C_FUNCTION)
 #define IDENTIFIER_IS_CONSTANT(X) ((X) & IDENTIFIER_CONSTANT)
-#define IDENTIFIER_IS_VARIABLE(X) (!((X) & (IDENTIFIER_FUNCTION | IDENTIFIER_CONSTANT)))
+#define IDENTIFIER_IS_VARIABLE(X) (!((X) & IDENTIFIER_TYPE_MASK))
 #define IDENTIFIER_IS_ALIAS(X)	((X) & IDENTIFIER_ALIAS)
 
 #define IDENTIFIER_MASK 255
@@ -260,39 +280,135 @@ struct program_constant
 /*
  * All identifiers in this program
  * and all identifiers in inherited programs
- * needs to have a 'struct reference' in this
+ * need to have a 'struct reference' in this
  * program. When we overload a function, we simply
  * change the reference to point to the new 'struct identifier'.
+ *
+ * When an identifier is represented as an integer, it's typically the
+ * offset of the corresponding struct reference in
+ * program.identifier_references.
  */
 struct reference
 {
+  /* Offset of the struct inherit in program.inherits for the program
+   * that the struct identifier is in. See INHERIT_FROM_PTR and
+   * INHERIT_FROM_INT. */
   unsigned INT16 inherit_offset;
+
+  /* Offset of the struct identifier in program.identifiers in the
+   * program pointed to by the struct inherit through inherit_offset
+   * above. See ID_FROM_PTR and ID_FROM_INT. */
   unsigned INT16 identifier_offset;
-  INT16 id_flags; /* static, private etc.. */
+
+  /* ID_* flags - static, private etc.. */
+  INT16 id_flags;
 };
 
 /* Magic value used as identifier reference integer to refer to this. */
 #define IDREF_MAGIC_THIS -1
 
+/* Magic values in inherit.parent_offset; see below. */
+#define OBJECT_PARENT -18
+#define INHERIT_PARENT -17
+
 /*
  * Each program has an array of these,
  * the first entry points to itself, the
  * rest are from inherited programs.
  * Note that when a program is inherited,
- * all 'struct inherit' from that program is
+ * all 'struct inherit' from that program are
  * copied, so the whole tree of inherits is
  * represented.
  */
 struct inherit
 {
+  /* The depth of the inherited program in this program. I.e. the
+   * number of times the program has been inherited, directly or
+   * indirectly.
+   *
+   * Note that the struct inherit for the program that directly
+   * inherited the program represented by this struct inherit can be
+   * found by going backwards in program.inherits from this struct
+   * until one is found with an inherit_level less than this one. */
   INT16 inherit_level;
+
+  /* All the identifier references in the inherited program has been
+   * copied to this program with the first one at this offset. */
   INT16 identifier_level;
+
+  /* The index of the identifier reference in the parent program for
+   * the identifier from which this inherit was done. -1 if there's no
+   * such thing. It's always -1 in the inherit struct for the top
+   * level program. */
   INT16 parent_identifier;
+
+  /* Describes how to find the parent object for the external
+   * identifier references associated with this inherit:
+   *
+   * OBJECT_PARENT: Follow the object parent, providing
+   *   PROGRAM_USES_PARENT is set in the program containing this
+   *   inherit. See PARENT_INFO. This is used for external references
+   *   in the top level program (i.e. the one containing the inherit
+   *   table).
+   *
+   * INHERIT_PARENT: Follow the parent pointer in this inherit. This
+   *   is used when finished programs with parent objects are
+   *   inherited.
+   *
+   * A non-negative integer: The parent is found by following this
+   *   number of parent pointers in the program that directly
+   *   inherited the program in this inherit, i.e. in the closest
+   *   lower level inherit. This is used when a program is inherited
+   *   whose parent is still being compiled, so it's parent object is
+   *   fake. That implies that that program also contains the current
+   *   program on some level, and that level is stored here. An
+   *   example:
+   *
+   *  	 class A {
+   *  	   class B {}
+   *  	   class C {
+   *  	     class D {
+   *  	       inherit B;
+   *  	     }
+   *  	   }
+   *  	 }
+   *
+   *   The parent program of B is A, which is still being compiled
+   *   when B is inherited in D, so it has a fake object. A is also
+   *   the parent of D, but two levels out, and hence 2 is stored in
+   *   parent_offset.
+   *
+   *   Note that parent_offset can be 0:
+   *
+   *     class A {
+   *       class B {}
+   *       inherit B;
+   *     }
+   */
   INT16 parent_offset;
+
+  /* The offset of the first entry in prog->identifier_references that
+   * comes from the program in this inherit. prog is in this case the
+   * program that directly inherited it, and not the top level
+   * program. I.e. for inherits on level 1, this is always the same as
+   * identifier_level.
+   *
+   * Are both really necessary? /mast */
   size_t identifier_ref_offset;
+
+  /* Offset in object->storage to the start of the storage for the
+   * variables from the program in this inherit. */
   ptrdiff_t storage_offset;
+
+  /* The parent object for the program in this inherit, or NULL if
+   * there isn't any. */
   struct object *parent;
+
+  /* The program for this inherit. */
   struct program *prog;
+
+  /* The name of the inherit, if there is any. For nested inherits,
+   * this can be a string on the form "A::B::C". */
   struct pike_string *name;
 };
 
@@ -334,7 +450,8 @@ struct pike_trampoline
 /* Objects created from this program are constant and shareable */
 #define PROGRAM_CONSTANT 0x40
 
-/* */
+/* Objects have pointers to the parent object. Use LOW_PARENT_INFO or
+ * PARENT_INFO to extract it. */
 #define PROGRAM_USES_PARENT 0x80
 
 /* Objects should not be destructed even when they only have weak
@@ -350,7 +467,8 @@ struct pike_trampoline
 /* Program has not yet been used for compilation */
 #define PROGRAM_VIRGIN 0x800
 
-/* */
+/* Don't allow the program to be inherited or cloned if there's no
+ * parent object. Only set if PROGRAM_USES_PARENT is. */
 #define PROGRAM_NEEDS_PARENT 0x1000
 
 /* Using define instead of enum allows for ifdefs - Hubbe */
@@ -401,7 +519,7 @@ struct program
   struct program *prev;
   struct program *parent;
   
-  struct node_s *(*optimize)(struct node_s *n);
+  node *(*optimize)(node *n);
 
   void (*event_handler)(int);
 #ifdef PIKE_DEBUG
@@ -413,26 +531,50 @@ struct program
 
   size_t total_size;
 
-#define FOO(NUMTYPE,TYPE,NAME) TYPE * NAME ;
+#define FOO(NUMTYPE,TYPE,ARGTYPE,NAME) TYPE * NAME ;
 #include "program_areas.h"
 
-#define FOO(NUMTYPE,TYPE,NAME) NUMTYPE PIKE_CONCAT(num_,NAME) ;
+#define FOO(NUMTYPE,TYPE,ARGTYPE,NAME) NUMTYPE PIKE_CONCAT(num_,NAME) ;
 #include "program_areas.h"
   
   INT16 lfuns[NUM_LFUNS];
 };
 
+#if 0
+static inline int CHECK_IDREF_RANGE (int x, const struct program *p)
+{
+  if (x < 0 || x >= p->num_identifier_references)
+    debug_fatal ("Identifier reference index %d out of range 0..%d\n", x,
+		 p->num_identifier_references - 1);
+  return x;
+}
+#else
+#define CHECK_IDREF_RANGE(X, P) (X)
+#endif
+
+#define PTR_FROM_INT(P, X)	((P)->identifier_references + \
+				 CHECK_IDREF_RANGE((X), (P)))
+
 #define INHERIT_FROM_PTR(P,X) (dmalloc_touch(struct program *,(P))->inherits + (X)->inherit_offset)
 #define PROG_FROM_PTR(P,X) (dmalloc_touch(struct program *,INHERIT_FROM_PTR(P,X)->prog))
 #define ID_FROM_PTR(P,X) (PROG_FROM_PTR(P,X)->identifiers+(X)->identifier_offset)
-#define INHERIT_FROM_INT(P,X) INHERIT_FROM_PTR(P,(P)->identifier_references+(X))
-#define PROG_FROM_INT(P,X) PROG_FROM_PTR(P,(P)->identifier_references+(X))
-#define ID_FROM_INT(P,X) ID_FROM_PTR(P,(P)->identifier_references+(X))
+#define INHERIT_FROM_INT(P,X) INHERIT_FROM_PTR(P, PTR_FROM_INT(P, X))
+#define PROG_FROM_INT(P,X) PROG_FROM_PTR(P, PTR_FROM_INT(P, X))
+#define ID_FROM_INT(P,X) ID_FROM_PTR(P, PTR_FROM_INT(P, X))
 
 #define FIND_LFUN(P,N) ( dmalloc_touch(struct program *,(P))->flags & PROGRAM_FIXED?((P)->lfuns[(N)]):low_find_lfun((P), (N)) )
 #define QUICK_FIND_LFUN(P,N) (dmalloc_touch(struct program *,(P))->lfuns[N])
 
-#define free_program(p) do{ struct program *_=(p); debug_malloc_touch(_); if(!sub_ref(_)) really_free_program(_); }while(0)
+#define free_program(p) do{						\
+    struct program *_=(p);						\
+    debug_malloc_touch(_);						\
+    DO_IF_DEBUG (							\
+      DO_IF_PIKE_CLEANUP (						\
+	if (gc_external_refs_zapped)					\
+	  gc_check_zapped (_, PIKE_T_PROGRAM, __FILE__, __LINE__)));	\
+    if(!sub_ref(_))							\
+      really_free_program(_);						\
+  }while(0)
 
 BLOCK_ALLOC_FILL_PAGES(program, n/a)
 
@@ -459,10 +601,9 @@ extern int compilation_depth;
 #define ADD_STORAGE(X) low_add_storage(sizeof(X), ALIGNOF(X),0)
 #define STORAGE_NEEDED(X) ((X)->storage_needed - (X)->inherits[0].storage_offset)
 
-#define FOO(NUMTYPE,TYPE,NAME) void PIKE_CONCAT(add_to_,NAME(TYPE ARG));
+#define FOO(NUMTYPE,TYPE,ARGTYPE,NAME) void PIKE_CONCAT(add_to_,NAME(ARGTYPE ARG));
 #include "program_areas.h"
 
-
 typedef int supporter_callback (void *, int);
 struct Supporter
 {
@@ -482,17 +623,17 @@ struct Supporter
 
 /* Prototypes begin here */
 void ins_int(INT32 i, void (*func)(char tmp));
-void ins_short(INT16 i, void (*func)(char tmp));
+void ins_short(int i, void (*func)(char tmp));
 void add_relocated_int_to_program(INT32 i);
 void use_module(struct svalue *s);
 void unuse_modules(INT32 howmany);
-struct node_s *find_module_identifier(struct pike_string *ident,
-				      int see_inherit);
-struct node_s *resolve_identifier(struct pike_string *ident);
-struct node_s *program_magic_identifier (struct program_state *state,
-					 int state_depth, int inherit_num,
-					 struct pike_string *ident,
-					 int colon_colon_ref);
+node *find_module_identifier(struct pike_string *ident,
+			     int see_inherit);
+node *resolve_identifier(struct pike_string *ident);
+node *program_magic_identifier (struct program_state *state,
+				int state_depth, int inherit_num,
+				struct pike_string *ident,
+				int colon_colon_ref);
 struct program *parent_compilation(int level);
 struct program *id_to_program(INT32 id);
 void optimize_program(struct program *p);
@@ -523,6 +664,9 @@ PMOD_EXPORT void set_gc_recurse_callback(void (*m)(struct object *));
 PMOD_EXPORT void set_gc_check_callback(void (*m)(struct object *));
 void pike_set_prog_event_callback(void (*cb)(int));
 void pike_set_prog_optimize_callback(node *(*opt)(node *));
+int really_low_reference_inherited_identifier(struct program_state *q,
+					      int e,
+					      int i);
 int low_reference_inherited_identifier(struct program_state *q,
 				       int e,
 				       struct pike_string *name,
@@ -540,9 +684,7 @@ void low_inherit(struct program *p,
 PMOD_EXPORT void do_inherit(struct svalue *s,
 		INT32 flags,
 		struct pike_string *name);
-void compiler_do_inherit(node *n,
-			 INT32 flags,
-			 struct pike_string *name);
+void compiler_do_inherit(node *n, INT32 flags, struct pike_string *name);
 int call_handle_inherit(struct pike_string *s);
 void simple_do_inherit(struct pike_string *s,
 		       INT32 flags,
@@ -578,32 +720,36 @@ PMOD_EXPORT int simple_add_constant(const char *name,
 			struct svalue *c,
 			INT32 flags);
 PMOD_EXPORT int add_integer_constant(const char *name,
-			 INT32 i,
-			 INT32 flags);
+				     INT_ARG_TYPE i,
+				     INT32 flags);
 PMOD_EXPORT int quick_add_integer_constant(const char *name,
-			       int name_length,
-			       INT32 i,
-			       INT32 flags);
+					   int name_length,
+					   INT_ARG_TYPE i,
+					   INT32 flags);
 PMOD_EXPORT int add_float_constant(const char *name,
-			 double f,
-			 INT32 flags);
+				   FLOAT_ARG_TYPE f,
+				   INT32 flags);
+PMOD_EXPORT int quick_add_float_constant(const char *name,
+					 int name_length,
+					 FLOAT_ARG_TYPE f,
+					 INT32 flags);
 PMOD_EXPORT int add_string_constant(const char *name,
-			const char *str,
-			INT32 flags);
+				    const char *str,
+				    INT32 flags);
 PMOD_EXPORT int add_program_constant(const char *name,
 			 struct program *p,
 			 INT32 flags);
 PMOD_EXPORT int add_object_constant(const char *name,
 			struct object *o,
 			INT32 flags);
-PMOD_EXPORT int add_function_constant(const char *name, void (*cfun)(INT32), const char * type, INT16 flags);
+PMOD_EXPORT int add_function_constant(const char *name, void (*cfun)(INT32), const char * type, int flags);
 PMOD_EXPORT int debug_end_class(const char *name, ptrdiff_t namelen, INT32 flags);
 INT32 define_function(struct pike_string *name,
 		      struct pike_type *type,
-		      unsigned INT16 flags,
-		      unsigned INT8 function_flags,
+		      unsigned flags,
+		      unsigned function_flags,
 		      union idptr *func,
-		      unsigned INT16 opt_flags);
+		      unsigned opt_flags);
 int really_low_find_shared_string_identifier(struct pike_string *name,
 					     struct program *prog,
 					     int flags);
@@ -631,14 +777,21 @@ PMOD_EXPORT struct pike_string *low_get_program_line(struct program *prog,
 						     INT32 *linep);
 PMOD_EXPORT struct pike_string *get_program_line(struct program *prog,
 						 INT32 *linep);
-char *debug_get_line(PIKE_OPCODE_T *pc, struct program *prog, INT32 *linep);
+PMOD_EXPORT char *low_get_program_line_plain (struct program *prog, INT32 *linep,
+					      int malloced);
 PMOD_EXPORT struct pike_string *low_get_line(PIKE_OPCODE_T *pc,
 					     struct program *prog, INT32 *linep);
+PMOD_EXPORT char *low_get_line_plain (PIKE_OPCODE_T *pc, struct program *prog,
+				      INT32 *linep, int malloced);
 PMOD_EXPORT struct pike_string *get_line(PIKE_OPCODE_T *pc,
 					 struct program *prog, INT32 *linep);
-void my_yyerror(char *fmt,...)  ATTRIBUTE((format(printf,1,2)));
+PMOD_EXPORT struct pike_string *low_get_function_line (struct object *o,
+						       int fun, INT32 *linep);
+void va_yyerror(const char *fmt, va_list args);
+void my_yyerror(const char *fmt,...)  ATTRIBUTE((format(printf,1,2)));
 struct pike_string *format_exception_for_error_msg (struct svalue *thrown);
-void yy_describe_exception(struct svalue *thrown);
+void handle_compile_exception (const char *yyerror_fmt, ...)
+  ATTRIBUTE((format(printf,1,2)));
 struct supporter_marker;
 void verify_supporters(void);
 void init_supporter(struct Supporter *s,
@@ -655,15 +808,15 @@ struct program *compile(struct pike_string *aprog,
 			struct program *atarget,
 			struct object *aplaceholder);
 PMOD_EXPORT int pike_add_function2(const char *name, void (*cfun)(INT32),
-				   const char *type, unsigned INT8 flags,
-				   unsigned INT16 opt_flags);
+				   const char *type, unsigned flags,
+				   unsigned opt_flags);
 PMOD_EXPORT int quick_add_function(const char *name,
 				   int name_length,
 				   void (*cfun)(INT32),
 				   const char *type,
 				   int type_length,
-				   unsigned INT8 flags,
-				   unsigned INT16 opt_flags);
+				   unsigned flags,
+				   unsigned opt_flags);
 void check_all_programs(void);
 void placeholder_index(INT32 args);
 void init_program(void);
@@ -675,7 +828,7 @@ void gc_check_all_programs(void);
 void gc_mark_all_programs(void);
 void gc_cycle_check_all_programs(void);
 void gc_zap_ext_weak_refs_in_programs(void);
-void gc_free_all_unreferenced_programs(void);
+size_t gc_free_all_unreferenced_programs(void);
 void push_compiler_frame(int lexical_scope);
 void low_pop_local_variables(int level);
 void pop_local_variables(int level);
@@ -692,7 +845,8 @@ void yywarning(char *fmt, ...) ATTRIBUTE((format(printf,1,2)));
 struct implements_cache_s;
 PMOD_EXPORT int implements(struct program *a, struct program *b);
 PMOD_EXPORT int is_compatible(struct program *a, struct program *b);
-int yyexplain_not_implements(struct program *a, struct program *b, int flags);
+void yyexplain_not_compatible(struct program *a, struct program *b, int flags);
+void yyexplain_not_implements(struct program *a, struct program *b, int flags);
 PMOD_EXPORT void *parent_storage(int depth);
 PMOD_EXPORT void change_compiler_compatibility(int major, int minor);
 void make_program_executable(struct program *p);
@@ -723,6 +877,9 @@ void count_memory_in_programs(INT32*,INT32*);
 #define ADD_INT_CONSTANT(NAME, CONST, FLAGS) \
   quick_add_integer_constant(NAME, CONSTANT_STRLEN(NAME), CONST, FLAGS)
 
+#define ADD_FLOAT_CONSTANT(NAME, CONST, FLAGS) \
+  quick_add_float_constant(NAME, CONSTANT_STRLEN(NAME), CONST, FLAGS)
+
 #define PIKE_MAP_VARIABLE(NAME, OFFSET, TYPE, RTTYPE, FLAGS) \
   quick_map_variable(NAME, CONSTANT_STRLEN(NAME), OFFSET, \
                      TYPE, CONSTANT_STRLEN(TYPE), RTTYPE, FLAGS)
