@@ -1,3 +1,10 @@
+/*
+|| This file is part of Pike. For copyright information see COPYRIGHT.
+|| Pike is distributed under GPL, LGPL and MPL. See the file COPYING
+|| for more information.
+|| $Id: queue.h,v 1.6 2002/10/11 01:39:37 nilsson Exp $
+*/
+
 #ifndef QUEUE_H
 #define QUEUE_H
 
